/*-------------------------------------------------------------------
Name......... demo.c
Author....... Marc Adams
Date......... 19/01/04
Description.. Console menu driven non-video demonstration program
............. using the Pluto 6 device drivers
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****   Include Files */
#include    "17269.h"
#include    "hardware.h"
#include    "peripherals.h"
#include    "interface.h"
#include    "config.h"

#include    "fpga.h"
#include    "demo.h"
#include    "demo.sp"


/****   Local Definitions */

#define BUFFER_SIZE 	20
#define DEFAULT_VOLUME	150

int atoi(const char *nptr);
int sprintf(char *str, const char *format, ...);
int tolower(int c);

/****   Global Variables */

/* vfd text scroll counters */
LONG output_string_pos = 0;
LONG input_string_pos = 0;

static LONG vfd_counter = VFD_ANIM_TIME;
static LONG drainOutputsCounter = DRAIN_OUTPUT_TIME;
static LONG leds_update_counter = LED_ANIM_TIME;
static LONG lamps_update_count= LAMP_ANIM_TIME;
static LONG serial_counter = SERIAL_ANIM_TIME;

/* Output position variables for auto demo */
static  BYTE    current_od = 16;
static  BYTE    previous_od = 16;

/* LED Animation mode */
static BYTE animationMode = 0;

/* Multiplexed outputs counter for auto demo */
static  BYTE    mpx_update = 0;

/* current lamp position */
static  WORD    current = 0;
static  BYTE    brightness;
static  BYTE    bright_index;

/****   Local Variables */

FILE_DESCRIPTOR HSFFile;
LONG  volume=DEFAULT_VOLUME;

/****   Global Constants */

/****   Local Constants */

/********************************************************************
*****   Interrupt Functions
********************************************************************/
/*-------------------------------------------------------------------
Name......... update_counter()
Author....... Marc Adams
Date......... 11/09/03
Description.. Interrupt function to run in the 1ms ISR. Decrements
............. the counters that activate the automated demo
............. functionality
Parameters... None.
Return....... None
History...... 13/01/04 - changed from 10ms to 1ms interrupt - MJB
-------------------------------------------------------------------*/

static void update_counter()
{
    if (vfd_counter) vfd_counter--;
    if (drainOutputsCounter) drainOutputsCounter--;
    if (leds_update_counter) leds_update_counter--;
    if (lamps_update_count) lamps_update_count--;
    if (serial_counter) serial_counter--;

}

/*******************************************************************
*****   Functions
********************************************************************/


/*-------------------------------------------------------------------
Name......... DemoFnc()
Author....... Marc Adams
Date......... 25/07/03
Description.. Continuous loop command parser function running the
............. top level menu
Parameters... None.
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

void        DemoFnc()
{
    const   DEVICE* serialdevice;
    BYTE    channel;
    CMD     command;

    /*
    Configure the Coldfire UART channel 0 used for the
    console menu
    */
    serialdevice=&SerialDevice;
    channel = 0;

    /* run startup sequence */
    startup(serialdevice, channel);

    /* Install Update Counters */
    InstallIntFnc(update_counter, Int1msLst);

    time_out_demo(serialdevice, channel);

    while(TRUE)
    {
        command = getcommand(commands, serialdevice, channel);

        switch(command)
        {
		case LAMPSCMD:
			demolamps(lampcommands, serialdevice, channel);
			break;

		case LEDSCMD:
			demoleds(ledcommands, serialdevice, channel);
			break;

		case INPUTSCMD:
			demoinputs(inputcommands, serialdevice, channel);
			break;

		case OUTPUTSCMD:
			demooutputs(serialdevice, channel);
			break;

		case REELCMD:
			demoreels(reelcommands, serialdevice, channel);
			break;

		case UARTCMD:
			demouarts(uartcommands, serialdevice, channel);
			break;

		case SOUNDCMD:
			demoaudio(audiocommands, serialdevice, channel);
			break;

		case VFD1CMD:
			demovfd(0, serialdevice, channel);
			break;

		case CSDCMD:
			democsd(csdcommands, serialdevice, channel);
			break;

		case E2ROMCMD:
			demoe2rom(e2romcommands, serialdevice, channel);
			break;
        }
    }
}

/*-------------------------------------------------------------------
Name......... startup()
Author....... Marc Adams
Date......... 25/07/03
Description.. Demonstration program startup sequence
Parameters... 1) Address of serial device used for the console menu
............. 2) Serial channel
Return....... None
History...... 11/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void startup(const DEVICE* serialdevice,BYTE channel)
{
    LONG index;

    /* Terminal intro message */
    TxString(serialdevice,channel,TITLE);
    TxString(serialdevice,channel,"\r");
    TxString(serialdevice,channel,"Drawing Number 81-");
    TxString(serialdevice,channel,DRAWING);
    TxString(serialdevice,channel,"-");
    TxString(serialdevice,channel,ISSUE);
    TxString(serialdevice,channel,"\r\n");

    ResetVFD1();

    /* Lamps display */
    SetDefaultBrightness();

    /* turn off leds*/
    for(index = 0;index < 16;index++)
    {
        LEDBits(digit_table0_15[index],ledchartable[index],OFF);
        LEDBits(digit_table16_31[index],ledchartable[index],OFF);
    }
}

/********************************************************************
*** Menu and Command Functions
********************************************************************/

/*-------------------------------------------------------------------
Name......... getcommand()
Author....... Marc Adams
Date......... 25/07/03
Description.. Read the command from the serial port. If there is no user
............. input for a given time run the automated demo code
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... Command number selected
History...... 25/07/03 - written to test driver updates  - MSA
			  11/08/05 - fixed invalid command - JC
-------------------------------------------------------------------*/

static BYTE getcommand(const BYTE* const* cmd,
            const DEVICE* serialdevice,BYTE channel)
{
    BYTE    str[BUFFER_SIZE];
    UBYTE   index;
    BYTE    ch;
    BYTE    command;
    TIMER   menutimer;

    /* Get services */
    ClearRx(serialdevice, channel);
    showcommands(cmd, serialdevice, channel);

    while(TRUE)
    {
        /* Prompt */
        index = 0;
        command = -1;
        TxString(serialdevice, channel, "\r>");

        /* set time out automated demo timer */
        SetTimer(&menutimer,2000);

        while(command<0)
        {
            /* get character */
            ch = RxChar(serialdevice, channel);

            while((ch<0) && (menutimer > 0))
            {
                ch = RxChar(serialdevice, channel);
            }

            if(menutimer == 0)
            {
                /* timer expired so run auto demo */
                time_out_demo(serialdevice, channel);

                /* redisplay commands */
                showcommands(cmd,serialdevice,channel);

                /* display prompt */
                TxString(serialdevice,channel,"\r>");

                /* reset timer */
                SetTimer(&menutimer,2000);

            }
            else
            {
                /* got character */
                /* reset timer */
                SetTimer(&menutimer,2000);

                if((ch!='\r') && (ch!='\b'))
                {
                    /* Echo character */
                    while(!TxChar(serialdevice,channel,ch));
                    /* store character in command string */
                    str[index++]=ch;
                    /* Exceeded buffer size so start again */
                    if(index==sizeof(str))
                    {
                        break;
                    }
                }
                else if(ch == '\b')
                {
                    /* check for start of string */
                    if(index>0 )
                    {
                        /*
                        send backspace to move cursor back
                        one character console
                        */
                        while(!TxChar(serialdevice,channel,ch));
                        /*
                        overwrite character at current cursor
                        position with a space
                        */
                        while(!TxChar(serialdevice,channel,' '));
                        /*
                        send backspace to move cursor back
                        to space character on console
                        */
                        while(!TxChar(serialdevice,channel,ch));

                        /*
                        decrement string pointer so that character
                        that has been deleted is overwritten by
                        next character entered */
                        index--;
                    }
                }
                else
                {
                    /* Echo character */
                    while(!TxChar(serialdevice,channel,ch));
                    /* Terminate string */
                    str[index]=0;
                    /* Identify command */
                    command = idcommand(cmd, str);

                    if(command>0)
                    {
                        /* Kill timer */
                        SetTimer(&menutimer,0);
                        /* Return valid command */
                        return(command-1);
                        break;
                    }

                    /* determine if an invalid command */

                    if (command == (BYTE)INVALID_COMMAND)
                    {
						/* clear the buffer */
						for (index = 0; index < BUFFER_SIZE; index++)
						{
							str[index] = 0;
						}

						/* clear index */
						index = 0;
					}
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... showcommands()
Author....... Marc Adams
Date......... 25/07/03
Description.. Display the available commands on the console
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void showcommands(const BYTE*const* cmd ,
                         const DEVICE* serialdevice,BYTE channel)
{
    LONG    index;

    /* Print commands */
    for(index = 0;cmd[index];index++)
    {
        TxString(serialdevice,channel,cmd[index]);
        while(!TxChar(serialdevice,channel,'\r')); /* Send CR */
    }
}

/*-------------------------------------------------------------------
Name......... idcommand()
Author....... Marc Adams
Date......... 25/07/03
Description.. Identify a command and return the command index
Parameters... 1) Commands  for the menu section
............. 2) Address of the string entered as a menu selection
Return....... Index number of the entered command
			  or invalid command failure
History...... 25/07/03 - written to test driver updates  - MSA
			  10/08/05 - fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE     idcommand(const BYTE*const* cmd,BYTE* str)
{
    BYTE* menustringptr;
    UBYTE  index;

    menustringptr = str;

    /* convert input string to lower case */
    while(*menustringptr)
    {
        *menustringptr++ = tolower(*menustringptr);
    }

    for(index=0; cmd[index]; index++)
    {
        if(!strcmp(cmd[index],str))
        {
            return(index);
        }
    }

	return INVALID_COMMAND;
}

/********************************************************************
*** Lamps
********************************************************************/
/*-------------------------------------------------------------------
Name......... demolamps()
Author....... Marc Adams
Date......... 30/07/03
Description.. Demonstrate the operation of the multiplexed lamps
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 30/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demolamps(const BYTE*const* cmd ,const DEVICE* serialdevice,
                      BYTE channel)
{
    const   LAMP* lp;
    LAMPCMD command;
    WORD    i,ch;
    BYTE    exit;
    STATUS  st;

    st=ON;

    SetDefaultBrightness();

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        case STROBECMD:
            /* Cycle through strobes */
            exit=FALSE;
            TxString(serialdevice,channel,"\rPress 'x' to exit...");
            TxString(serialdevice,channel,"\rPress anything  else to advance");
            while(!exit)
            {
                for(i=0;strobelst[i];i++)
                {
                    lp=strobelst[i];
                    while(*lp!=EOF_LAMPS)
                    {
                        Lamp(*lp++,st);
                    }
                    TxString(serialdevice,channel,"\rstrobe ");
                    while(!TxChar(serialdevice,channel,NibbleTable[i]));

                    while((ch=RxChar(serialdevice,channel))<0);
                    lp=strobelst[i];
                    while(*lp!=EOF_LAMPS) Lamp(*lp++,OFF);
                    if(tolower(ch)=='x')
                    {
                        TxString(serialdevice,channel,"\r\r ");
                        exit=TRUE;
                        break;
                    }
                }
            }
            break;


        case ROWCMD:
            /* Cycle through rows */
            exit=FALSE;
            TxString(serialdevice,channel,"\rPress 'x' to exit...");
            TxString(serialdevice,channel,"\rPress anything  else to advance");
            while(!exit)
            {
                for(i=0;rowlst[i];i++)
                {
                    lp=rowlst[i];
                    while(*lp!=EOF_LAMPS) Lamp(*lp++,st);
                    TxString(serialdevice,channel,"\rrow ");
                    while(!TxChar(serialdevice,channel,NibbleTable[i]));

                    while((ch=RxChar(serialdevice,channel))<0);
                    lp=rowlst[i];
                    while(*lp!=EOF_LAMPS) Lamp(*lp++,OFF);
                    if(tolower(ch)=='x')
                    {
                        TxString(serialdevice,channel,"\r\r ");
                        exit=TRUE;
                        break;
                    }
                }
            }
            break;

        case ONCMD:
            st=ON;
            break;

        case DIMCMD:
   /*         SetBrightness(LC0, 63);
            SetBrightness(LC1, 63);
            SetBrightness(LC2, 63);
            SetBrightness(LC3, 63);
            SetBrightness(LC4, 63);
            SetBrightness(LC5, 63);
            SetBrightness(LC6, 63);
            SetBrightness(LC7, 63);
            SetBrightness(LC8, 63);
            SetBrightness(LC9, 63);
            SetBrightness(LC10, 63);
            SetBrightness(LC11, 63);
            SetBrightness(LC12, 63);
            SetBrightness(LC13, 63);
            SetBrightness(LC14, 63);
            SetBrightness(LC15, 63);
   */
			SetBrightness(LC0, 0);
			SetBrightness(LC1, 0);
			SetBrightness(LC2, 0);
			SetBrightness(LC3, 0);
			SetBrightness(LC4, 0);
			SetBrightness(LC5, 0);
			SetBrightness(LC6, 0);
			SetBrightness(LC7, 0);
			SetBrightness(LC8, 0);
			SetBrightness(LC9, 0);
			SetBrightness(LC10, 0);
			SetBrightness(LC11, 0);
			SetBrightness(LC12, 0);
			SetBrightness(LC13, 0);
			SetBrightness(LC14, 0);
            SetBrightness(LC15, 0);

            st=ON & PERIOD0;
            break;

        case RESTORECMD:
            SetDefaultBrightness();
            break;

        case F1CMD:
            st=F1;
            break;

        case FF1CMD:
            st=FF1;
            break;

        case EXITLPCMD:
            return;

        }
    }
}

/********************************************************************
*** Leds
********************************************************************/
/*-------------------------------------------------------------------
Name......... demoleds()
Author....... Marc Adams
Date......... 05/08/03
Description.. Demonstrate the operation of the multiplexed leds
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 05/08/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demoleds(const BYTE*const* cmd ,const DEVICE* serialdevice,
                     BYTE channel)
{
    LONG index;
    LEDCMD  command;
    WORD    j,k;
    BYTE    exit;
    WORD    ch;
    BYTE led_digit_index;
    BYTE led_char_index;

    LAMP test;

    test = LAMP(LC0,LR0);

    led_digit_index = 0;
    led_char_index = 0;

    SetDefaultBrightness();

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);

        switch(command)
        {
		case DIGITCMD:
			exit=FALSE;
			TxString(serialdevice,channel,"\rPress 'x' to exit...");
			TxString(serialdevice,channel,"\rPress anything  else to advance");

			while(!exit)
			{
				for(index = 0;digit_table0_15[index] != EOF_DIGITS; index++)
				{
					if(!exit)
					{
						LEDBits(digit_table0_15[index],0xff,ON);

						TxString(serialdevice,channel,"\rdigit ");

						while(!TxChar(serialdevice,channel,NibbleTable[index]));

						while((ch=RxChar(serialdevice,channel))<0);

						LEDBits(digit_table0_15[index],0xff,OFF);
						if(ch=='x')
						{
							TxString(serialdevice,channel,"\r\r ");
							exit=TRUE;
						}
					}
					else
					{
						break;
					}
				}


				for(index = 0;digit_table16_31[index] != EOF_DIGITS; index++)
				{
					if(!exit)
					{
						LEDBits(digit_table16_31[index],0xff,ON);

						TxString(serialdevice,channel,"\rdigit ");

						while(!TxChar(serialdevice,channel,NibbleTable[index]));

						while((ch=RxChar(serialdevice,channel))<0);

						LEDBits(digit_table16_31[index],0xff,OFF);
						if(ch=='x')
						{
							TxString(serialdevice,channel,"\r\r ");
							exit=TRUE;
						}
					}
					else
					{
						break;
					}
				}
			}

			break;

        case SEGMENTCMD:
            exit=FALSE;
            TxString(serialdevice,channel,"\rPress 'x' to exit...");
            TxString(serialdevice,channel,"\rPress anything  else to advance");
            while(!exit)
            {
                for(j=1,k=0;j<0x100;j<<=1,k++)
                {
                    for(index=0;digit_table0_15[index]!=EOF_DIGITS;index++)
                    {
                        LEDBits(digit_table0_15[index],j,ON);
                        LEDBits(digit_table16_31[index],j,ON);
                    }
                    TxString(serialdevice,channel,"\rsegment ");
                    while(!TxChar(serialdevice,channel,'A'+k));

                    while((ch=RxChar(serialdevice,channel))<0);
                    for(index=0;digit_table0_15[index]!=EOF_DIGITS;index++)
                    {
                        LEDBits(digit_table0_15[index],j,OFF);
                        LEDBits(digit_table16_31[index],j,OFF);
                    }
                    if(ch=='x')
                    {
                        TxString(serialdevice,channel,"\r\r ");
                        exit=TRUE;
                        break;
                    }
                }
            }
            break;

        case DIMLEDCMD:
            SetBrightness(DIG0, 10);
            SetBrightness(DIG1, 10);
            SetBrightness(DIG2, 10);
            SetBrightness(DIG3, 10);
            SetBrightness(DIG4, 10);
            SetBrightness(DIG5, 10);
            SetBrightness(DIG6, 10);
            SetBrightness(DIG7, 10);
            SetBrightness(DIG8, 10);
            SetBrightness(DIG9, 10);
            SetBrightness(DIG10, 10);
            SetBrightness(DIG11, 10);
            SetBrightness(DIG12, 10);
            SetBrightness(DIG13, 10);
            SetBrightness(DIG14, 10);
            SetBrightness(DIG15, 10);
            break;

        case RESTORELEDCMD:
            SetDefaultBrightness();
            break;

        case EXITLEDCMD:
            return;

        }
    }
}

/********************************************************************
*** System Inputs
********************************************************************/

/*-------------------------------------------------------------------
Name......... demoinputs()
Author....... Marc Adams
Date......... 29/07/03
Description.. Demonstrate the system inputs
Parameters... 1) Commands for the menu section(not used)
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 29/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demoinputs(const BYTE*const* cmd ,const DEVICE* serialdevice,
                       BYTE channel)
{
    INPUTCMD    command;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
		case SYSTEMINPUTCMD:
			/* inputs 0 -31 */
			demosysinputs( serialdevice, channel );
			break;

		case METERSENSECMD:
			/* Current sense */
			demosense( 0, serialdevice, channel );
			break;

		case PUSHBUTTONCMD:
			/* Push button */
			demopushbutton( 0, serialdevice, channel );
			break;

		case EXITINPUTSCMD:
			return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... demosense()
Author....... Marc Adams
Date......... 29/07/03
Description.. Demonstrate the meter current sense circuit
Parameters... 1) Commands for the menu section(not used)
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 29/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demosense(const BYTE*const* cmd, const DEVICE* serialdevice,
                          BYTE channel)
{
    while(RxChar(serialdevice, channel) == -1)
    {

        if(CurrentSense())
        {
            while(!TxChar(serialdevice,channel,'1')); /* TRUE - echo 1 */
        }
        else
        {
            while(!TxChar(serialdevice,channel,'0')); /* FALSE - echo 0 */
        }

        while(!TxChar(serialdevice,channel,'\r'));
        while(!TxChar(serialdevice,channel,'\n'));

        Wait(100);
    }
}

/*-------------------------------------------------------------------
Name......... demopushbutton()
Author....... Marc Adams
Date......... 29/07/03
Description.. Demonstrate the operation of the push button input
Parameters... 1) Commands for the menu section(not used)
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 29/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demopushbutton(const BYTE*const* cmd,
                           const DEVICE* serialdevice, BYTE channel)
{
    while(RxChar(serialdevice, channel) == -1)
    {

        if ( PushButton() )
        {
            while(!TxChar(serialdevice,channel,'1')); /* TRUE - echo 1 */
        }
        else
        {
            while(!TxChar(serialdevice,channel,'0')); /* FALSE - echo 0 */
        }

        while(!TxChar(serialdevice,channel,'\r'));
        while(!TxChar(serialdevice,channel,'\n'));

        Wait(100);
    }
}

/*-------------------------------------------------------------------
Name......... demosysinputs()
Author....... Marc Adams
Date......... 29/07/03
Description.. Demonstrate input switches 0-31
Parameters... 1) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 29/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demosysinputs( const DEVICE* serialdevice, BYTE channel )
{
    LONG    index;
    BYTE    str[64];
    BYTE*   s;
    TIMER   Timer;

    while(TRUE)
    {

        /* Read input switches */
        for(index = 0;inputtable[index]!=EOF_INPUTS;index++)
        {
            if(Test(inputtable[index]))
            {
				str[index] = '1';
			}
            else
            {
				str[index] = '0';
			}
        }

        /* Terminate string */
        str[index] = 0;

        s = str;

        index = 8;

        /* Print input switch status */
        while(*s)
        {
            while(!TxChar(serialdevice,channel,*s));
            s++;
            if(--index == 0)
            {
                while(!TxChar(serialdevice,channel,' '));
                index = 8;
            }
        }
        while(!TxChar(serialdevice,channel,'\r'));

        /* Delay before next sample */
        SetTimer(&Timer,100);
        while(Timer)
        if(RxChar(serialdevice,channel)>=0)
        {
            /* Remove from timer list before deallocating from stack */
            SetTimer(&Timer,0);
            return;
        }
    }
}

/********************************************************************
*** OUTPUTS
********************************************************************/

/*-------------------------------------------------------------------
Name......... demooutputs()
Author....... Marc Adams
Date......... 29/07/03
Description.. Demonstrate the operation of the Pluto 6 outputs
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 04/08/03 - written to test driver updates  - MSA
			  10/08/05 - fixed problems with warnings - JC
-------------------------------------------------------------------*/

static void demooutputs( const DEVICE* serialdevice, BYTE channel )
{
    BYTE ch;
    BYTE outputstring[8];
    UBYTE stringindex;
    OUTPUT output;

    TxString(serialdevice, channel,"\rEnter output number");
    TxString(serialdevice, channel,"\ror x to exit...");

    while(TRUE)
    {
        output = -1;
        stringindex = 0;

        TxString(serialdevice, channel,"\rOP>");

        while ( output < 0 )
        {
            /* Wait for char to arrive */
            while( ( ch=RxChar( serialdevice, channel ) ) <0 );

            /* Echo character */
            while( !TxChar( serialdevice, channel, ch ) );

            if(tolower(ch)=='x')
            {
                TxString(serialdevice, channel,"\r");
                return;
            }

            if( ch!='\r' )
            {
                outputstring[stringindex++] = ch;
                if( stringindex == sizeof( outputstring ) )
                {
                    /* Exceeded buffer size so start again */
                    break;
				}
            }
            else
            {
                outputstring[stringindex] = 0;
                output = atoi( outputstring );
                if((output>=16) &
                   (output<(sizeof(outputtable)/sizeof(outputtable[0]))))
                {
                    /* Drive output */
                    OutputOn(outputtable[output]);

                    TxString(serialdevice, channel,"Press any key to switch off\r");
                    TxString(serialdevice, channel,"or x to exit..\r");

                    /* Wait for char to arrive */
                    while((ch=RxChar(serialdevice, channel))<0);

                    /* Turn output off */
                    OutputOff(outputtable[output]);

                    break;
                }
                else
                {
                    TxString(serialdevice, channel,"Number out of range\r");
                    TxString(serialdevice, channel,"OP range 16..63 or x to exit\r");
                }
            }
        }
    }
}

/********************************************************************
*** REELS
********************************************************************/

/*-------------------------------------------------------------------
Name......... demoreels()
Author....... Marc Adams
Date......... 02/12/03
Description.. Demonstrate the operation of the Pluto 6 reel driver
Parameters... 1) command from user
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 02/12/03 - written to test driver updates  - MSA
			  10/08/05 - Deleted unused variable - JC
-------------------------------------------------------------------*/

static void demoreels(const BYTE*const* cmd ,const DEVICE* serialdevice,
                      BYTE channel)
{
    RLCMD   command;
    WORD    i,j;
    LONG    rmask;
    BYTE*   s;

    SetDeviceErrorMask(ERR_BADOPTO | ERR_BADREEL,&StepperDevice);
    TxString(serialdevice,channel,"\rInitialising reels...\r");
    InitialiseStepMotors(&StepperDevice);
    rmask=DetectStepMotorOptos(&StepperDevice);
    i=GetNofMotors(&StepperDevice);
    while(i--)
    {
        if(rmask & 1)
        {
			s=" OK ";
		}
        else
        {
			s="BAD ";
		}
        TxString(serialdevice,channel,s);
        rmask>>=1;
    }
    while(!TxChar(serialdevice,channel,'\r'));

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
		case INITCMD:
			TxString(serialdevice,channel,"\rInitialising reels...\r");
			InitialiseStepMotors(&StepperDevice);
			rmask=DetectStepMotorOptos(&StepperDevice);
			i=GetNofMotors(&StepperDevice);
			while(i--)
			{
				if(rmask & 1)
				{
					s=" OK ";
				}
				else
				{
					s="BAD ";
				}
				TxString(serialdevice,channel,s);
				rmask>>=1;
			}
			while(!TxChar(serialdevice,channel,'\r'));
			break;

		case STEP1CMD:
			SpinStepMotor(&StepperDevice,0,3,RAMP_NUDGE1);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case STEP2CMD:
			SpinStepMotor(&StepperDevice,1,3,RAMP_NUDGE1);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case STEP3CMD:
			SpinStepMotor(&StepperDevice,2,3,RAMP_NUDGE1);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case STEP4CMD:
			SpinStepMotor(&StepperDevice,3,3,RAMP_NUDGE1);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case SPINCMD:
			for(i=0;i<4;i++) SpinStepMotor(&StepperDevice,i,48+15*i,RAMP_96);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case SKILLCMD:
			for(i=0;i<4;i++) SpinStepMotor(&StepperDevice,i,48+15*i,RAMP_SKILLSLOW);
			while(RxChar(serialdevice,channel)<0);
			StopStepMotor(&StepperDevice,0);
			while(RxChar(serialdevice,channel)<0);
			StopStepMotor(&StepperDevice,1);
			while(RxChar(serialdevice,channel)<0);
			StopStepMotor(&StepperDevice,2);
			while(RxChar(serialdevice,channel)<0);
			StopStepMotor(&StepperDevice,3);
			while(!StepMotorsIdle(&StepperDevice));
			break;

		case STATUSCMD:
			for(i=0;i<4;i++)
			{
				j=GetStepMotorTabsMissed(&StepperDevice,i);
				while(!TxChar(serialdevice,channel,'T'));
				while(!TxChar(serialdevice,channel,'0'+j));
				j=GetStepMotorPosnCorrected(&StepperDevice,i);
				while(!TxChar(serialdevice,channel,'C'));
				while(!TxChar(serialdevice,channel,'0'+j));
				while(!TxChar(serialdevice,channel,' '));
				ClearStepMotorErrors(&StepperDevice,i);
			}
			while(!TxChar(serialdevice,channel,'\r'));
			break;


		case EXITRLCMD:
			TxString(serialdevice,channel,"\r\r");
			return;
        }
    }
}

/********************************************************************
*** UARTS
********************************************************************/

/*-------------------------------------------------------------------
Name......... demouarts()
Author....... Marc Adams
Date......... 01/09/03
Description.. Top level menu for the Coldfire serial module,
............. FPGA UARTS, and the DUART
............. Note: Coldfire UART channel 0 is used for the console
............. menu through the BACTA port and therefore does not
............. appear as a selectable menu item
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
			  10/08/05 - added default to switch statement - JC
-------------------------------------------------------------------*/

static  void demouarts(const BYTE*const* cmd ,
                       const DEVICE* serialdevice,BYTE channel)
{
    CMD command;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* Coldfire UART channel 1 */
        case CFUART1CMD:
            democfuart1(cfuart1commands, serialdevice, channel);
            break;

        /* FGPA UART 0 */
        case FPGAUART0CMD:
            demofpgauart0(fpgauart0commands, serialdevice, channel);
            break;

        /* FGPA UART 1 */
        case FPGAUART1CMD:
            demofpgauart1(fpgauart1commands, serialdevice, channel);
            break;

        /* DUART channel 0 */
        case DUART0CMD:
            demoduart0(duart0commands, serialdevice, channel);
            break;

        /* DUART channel 1 */
        case DUART1CMD:
            demoduart1(duart1commands, serialdevice, channel);
            break;

        default:
            return;


        }
    }
}

/*-------------------------------------------------------------------
Name......... democfuart1()
Author....... Marc Adams
Date......... 01/09/03
Description.. Demonstrate the operation of the Coldfire Serial module
............. channel 1
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void democfuart1(const BYTE*const* cmd ,
                        const DEVICE* serialdevice,BYTE channel)
{
    BYTE    baudsetting;
    BYTE    paritysetting;
    BYTE    databitssetting;
    BYTE    stopbitssetting;
    BYTE    routesetting;
    CFUART1TESTCMD  command;
    BYTE    character;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* configure the serial port */
        case CFCONFIG1CMD:
            baudsetting = getbaudrate(serialdevice, channel);
            paritysetting = getparity(serialdevice, channel);
            databitssetting = getdatabits(serialdevice, channel);
            stopbitssetting = getstopbits(serialdevice, channel);
            ConfigureSerial(&SerialDevice, 1, baudsetting, baudsetting,
                            databitssetting, stopbitssetting,
                            paritysetting, TXNONE, RXNONE);
            break;

        /* set the routing for the port */
        case CFROUTE1CMD:
            routesetting = getrouting(serialdevice, channel);
            SetRouting(&SerialDevice, 1, routesetting);
            break;

        /* transmit a string to test the port */
        case CFTRANS1CMD:
            TxString(&SerialDevice,1,"Transmitting on Coldfire UART1\r");
            break;

        case CFREC1CMD:

            character = 255;

            ClearRx(&SerialDevice, 1);
            TxString(&SerialDevice,1,"Enter characters or x to exit\r");
            TxString(&SerialDevice,1,">");

            while(TRUE)
            {
                if(character == 'x')
                {
                    TxString(&SerialDevice,1,"\r");
                    break;
                }

                while((character = RxChar(&SerialDevice, 1))<0);
                /* Echo character */
                while( !TxChar(&SerialDevice, 1, character ));
            }
            break;

        case CFEXITUART1CMD:
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... demofpgauart0()
Author....... Marc Adams
Date......... 01/09/03
Description.. Demonstrate the operation of FPGA UART 0
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demofpgauart0(const BYTE*const* cmd ,
                          const DEVICE* serialdevice,BYTE channel)
{
    BYTE    baudsetting;
    BYTE    paritysetting;
    BYTE    databitssetting;
    BYTE    stopbitssetting;
    BYTE    routesetting;
    FPGAUART0TESTCMD    command;
    BYTE    character;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* configure the serial port */
        case FPGACONFIG0CMD:
            baudsetting = getbaudrate(serialdevice, channel);
            paritysetting = getparity(serialdevice, channel);
            databitssetting = getdatabits(serialdevice, channel);
            stopbitssetting = getstopbits(serialdevice, channel);
            ConfigureSerial(&FPUART0Device, 0, baudsetting, baudsetting,
                            databitssetting, stopbitssetting,
                            paritysetting, TXNONE, RXNONE);
            break;

        /* set the routing for the port */
        case FPGAROUTE0CMD:
            routesetting = getrouting(serialdevice, channel);
            SetRouting(&FPUART0Device, 0, routesetting);
            break;

        /* transmit a string to test the port */
        case FPGATRANS0CMD:
            TxString(&FPUART0Device,0,"Transmitting on FPGA UART 0\r");
            break;

        case FPGAREC0CMD:

            character = 255;

            ClearRx(&FPUART0Device, 0);
            TxString(&FPUART0Device,0,"Enter characters or x to exit\r");
            TxString(&FPUART0Device,0,">");

            while(TRUE)
            {
                if(character == 'x')
                {
                    TxString(&FPUART0Device,0,"\r");
                    break;
                }

                while((character = RxChar(&FPUART0Device, 0))<0);
                /* Echo character */
                while( !TxChar(&FPUART0Device, 0, character ));
            }
            break;

        case FPGAEXITUART0CMD:
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... demofpgauart1()
Author....... Marc Adams
Date......... 01/09/03
Description.. Demonstrate the operation of FPGA UART 1
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demofpgauart1(const BYTE*const* cmd ,
                          const DEVICE* serialdevice,BYTE channel)
{
    BYTE    baudsetting;
    BYTE    paritysetting;
    BYTE    databitssetting;
    BYTE    stopbitssetting;
    BYTE    routesetting;
    FPGAUART1TESTCMD    command;
    BYTE    character;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* configure the serial port */
        case FPGACONFIG1CMD:
            baudsetting = getbaudrate(serialdevice, channel);
            paritysetting = getparity(serialdevice, channel);
            databitssetting = getdatabits(serialdevice, channel);
            stopbitssetting = getstopbits(serialdevice, channel);
            ConfigureSerial(&FPUART1Device, 0, baudsetting, baudsetting,
                            databitssetting, stopbitssetting,
                            paritysetting, TXNONE, RXNONE);
            break;

        /* set the routing for the port */
        case FPGAROUTE1CMD:
            routesetting = getrouting(serialdevice, channel);
            SetRouting(&FPUART1Device, 0, routesetting);
            break;

        /* transmit a string to test the port */
        case FPGATRANS1CMD:
            TxString(&FPUART1Device,0,"Transmitting on FPGA UART 1\r");
            break;

        case FPGAREC1CMD:

            character = 255;

            ClearRx(&FPUART1Device, 0);
            TxString(&FPUART1Device,0,"Enter characters or x to exit\r");
            TxString(&FPUART1Device,0,">");

            while(TRUE)
            {
                if(character == 'x')
                {
                    TxString(&FPUART1Device,0,"\r");
                    break;
                }

                while((character = RxChar(&FPUART1Device, 0))<0);
                /* Echo character */
                while( !TxChar(&FPUART1Device, 0, character ));
            }
            break;

        case FPGAEXITUART1CMD:
            return;
        }
    }
}


/*-------------------------------------------------------------------
Name......... demoduart0()
Author....... Marc Adams
Date......... 01/09/03
Description.. Demonstrate the operation of DUART channel 0
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demoduart0(const BYTE*const* cmd ,
                       const DEVICE* serialdevice,BYTE channel)
{
    BYTE    baudsetting;
    BYTE    paritysetting;
    BYTE    databitssetting;
    BYTE    stopbitssetting;
    BYTE    routesetting;
    DUART0TESTCMD   command;
    BYTE    character;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* configure the serial port */
        case DUARTCONFIG0CMD:
            baudsetting = getbaudrate(serialdevice, channel);
            paritysetting = getparity(serialdevice, channel);
            databitssetting = getdatabits(serialdevice, channel);
            stopbitssetting = getstopbits(serialdevice, channel);
            ConfigureSerial(&DUART0Device, 0, baudsetting, baudsetting,
                            databitssetting, stopbitssetting,
                            paritysetting, TXNONE, RXNONE);
            break;

        /* set the routing for the port */
        case DUARTROUTE0CMD:
            routesetting = getrouting(serialdevice, channel);
            SetRouting(&DUART0Device, 0, routesetting);
            break;

        /* transmit a string to test the port */
        case DUARTTRANS0CMD:
            TxString(&DUART0Device,0,"Transmitting on DUART channel 0\r");
            break;

        case DUARTREC0CMD:

            character = 255;

            ClearRx(&DUART0Device, 0);
            TxString(&DUART0Device,0,"Enter characters or x to exit\r");
            TxString(&DUART0Device,0,">");

            while(TRUE)
            {
                if(character == 'x')
                {
                    TxString(&DUART0Device,0,"\r");
                    break;
                }

                while((character = RxChar(&DUART0Device, 0))<0);
                /* Echo character */
                while( !TxChar(&DUART0Device, 0, character ));
            }
            break;

        case EXITDUART0CMD:
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... demoduart1()
Author....... Marc Adams
Date......... 01/09/03
Description.. Demonstrate the operation of DUART channel 1
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 01/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demoduart1(const BYTE*const* cmd ,
                       const DEVICE* serialdevice, BYTE channel)
{
    BYTE    baudsetting;
    BYTE    paritysetting;
    BYTE    databitssetting;
    BYTE    stopbitssetting;
    BYTE    routesetting;
    DUART1TESTCMD command;
    BYTE    character;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        /* configure the serial port */
        case DUARTCONFIG1CMD:
            baudsetting = getbaudrate(serialdevice, channel);
            paritysetting = getparity(serialdevice, channel);
            databitssetting = getdatabits(serialdevice, channel);
            stopbitssetting = getstopbits(serialdevice, channel);
            ConfigureSerial(&DUART0Device, 1, baudsetting, baudsetting,
                            databitssetting, stopbitssetting,
                            paritysetting, TXNONE, RXNONE);
            break;

        /* set the routing for the port */
        case DUARTROUTE1CMD:
            routesetting = getrouting(serialdevice, channel);
            SetRouting(&DUART0Device, 1, routesetting);
            break;

        /* transmit a string to test the port */
        case DUARTTRANS1CMD:
            TxString(&DUART0Device, 1,"Transmitting on DUART channel 1\r");
            break;

        case DUARTREC1CMD:

            character = 255;

            ClearRx(&DUART0Device, 1);
            TxString(&DUART0Device,1,"Enter characters or x to exit\r");
            TxString(&DUART0Device,1,">");

            while(TRUE)
            {
                if(character == 'x')
                {
                    TxString(&DUART0Device,1,"\r");
                    break;
                }

                while((character = RxChar(&DUART0Device, 1))<0);
                /* Echo character */
                while( !TxChar(&DUART0Device, 1, character ));
            }
            break;
        case EXITDUART1CMD:
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... getbaudrate()
Author....... Marc Adams
Date......... 01/09/03
Description.. Get the baud rate for the selected serial port
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... Defined value for the baud rate setting
History...... 01/09/03 - written to test driver updates  - MSA
			  10/08/05 - deleted used variable - JC
-------------------------------------------------------------------*/

static BYTE getbaudrate(const DEVICE* serialdevice,BYTE channel)
{
    BYTE    baudsetting[20];
    BYTE    ch;
    LONG    index;
    BYTE    valid;

    while(TRUE)
    {
        TxString(serialdevice, channel,"Enter Baud Rate\r");
        showcommands(baudrates, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        valid = -1;
        index = 0;

        while(valid < 0)
        {
            while((ch=RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel,ch));
                /* store character in command string */
                baudsetting[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(baudsetting))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back one
                    character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to space
                    character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*
                    decrement string pointer so that character
                    that has been deleted is overwritten by
                    next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                baudsetting[index] = 0;
                /* validate entry */
                valid = validate(baudrates, baudsetting);
                if(valid<0)
                {
                    TxString(serialdevice, channel,"Unsupported Baud Rate\r\n");
                    break;
                }
                else
                {
                    switch(valid)
                    {
                        case 0:
                            return BAUD19200;
                        case 1:
                            return BAUD9600;
                        case 2:
                            return BAUD4800;
                        case 3:
                            return BAUD2400;
                        case 4:
                            return BAUD1200;
                    }
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... getparity()
Author....... Marc Adams
Date......... 03/09/03
Description.. Get the parity settng for the selected serial port
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... Defined value for the parity setting
History...... 03/09/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE getparity(const DEVICE* serialdevice,BYTE channel)
{
    BYTE    paritysetting[10];
    BYTE    ch;
    UBYTE    index;
    BYTE    valid;

    while(TRUE)
    {
        TxString(serialdevice, channel,"Enter Parity\r");
        showcommands(parity, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        valid = -1;
        index = 0;

        while(valid < 0)
        {
            while((ch=RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel,ch));
                /* store character in command string */
                paritysetting[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(paritysetting))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back
                    one character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to
                    space character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*decrement string pointer so that
                    character that has been deleted is
                    overwritten by next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                paritysetting[index] = 0;
                /* validate entry */
                valid = validate(parity, paritysetting);
                if(valid<0)
                {
                    TxString(serialdevice, channel,"Invalid Parity Setting\r\n");
                    break;
                }
                else
                {
                    switch(valid)
                    {
                        case 0:
                            return PARITYNONE;
                        case 1:
                            return PARITYODD;
                        case 2:
                            return PARITYEVEN;
                    }
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... getdatabits()
Author....... Marc Adams
Date......... 03/09/03
Description.. Get the data bits settng for the selected serial port
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... Defined value for the data bits setting
History...... 03/09/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE getdatabits(const DEVICE* serialdevice, BYTE channel)
{
    BYTE    databitssetting[3];
    BYTE    ch;
    UBYTE   index;
    BYTE    valid;

    while(TRUE)
    {
        TxString(serialdevice, channel,"Enter Number of Data Bits\r");
        showcommands(databits, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        valid = -1;
        index = 0;

        while(valid < 0)
        {
            while((ch = RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel,ch));
                /* store character in command string */
                databitssetting[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(databitssetting))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back
                    one character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to
                    space character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*
                    decrement string pointer so that character
                    that has been deleted is overwritten by
                    next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                databitssetting[index] = 0;
                /* validate entry */
                valid = validate(databits, databitssetting);
                if(valid<0)
                {
                    TxString(serialdevice, channel,"Invalid Number of Data Bits\r\n");
                    break;
                }
                else
                {
                    switch(valid)
                    {
					case 0:
						return BITS7;
					case 1:
						return BITS8;
                    }
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... getstopbits()
Author....... Marc Adams
Date......... 03/09/03
Description.. Get the stop bits settng for the selected serial port
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... Defined value for the stop bits setting
History...... 03/09/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE getstopbits(const DEVICE* serialdevice,BYTE channel)
{
    BYTE    stopbitssetting[3];
    BYTE    ch;
    UBYTE   index;
    BYTE    valid;

    while(TRUE)
    {
        TxString(serialdevice, channel,"Enter Number of Stop Bits\r");
        showcommands(stopbits, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        valid = -1;
        index = 0;

        while(valid < 0)
        {
            while((ch = RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel, ch));
                /* store character in command string */
                stopbitssetting[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(stopbitssetting))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back
                    one character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to
                    space character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*
                    decrement string pointer so that character
                    that has been deleted is overwritten by
                    next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                stopbitssetting[index] = 0;
                /* validate entry */
                valid = validate(stopbits, stopbitssetting);
                if(valid<0)
                {
                    TxString(serialdevice, channel,"Invalid Number of Stop Bits\r\n");
                    break;
                }
                else
                {
                    switch(valid)
                    {
					case 0:
						return STOP1;
					case 1:
						return STOP2;
                    }
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... getrouting()
Author....... Marc Adams
Date......... 03/09/03
Description.. Get the routing settng for the selected serial port
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... Defined value for the routing setting
History...... 03/09/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE getrouting(const DEVICE* serialdevice, BYTE channel)
{
    BYTE    routesetting[10];
    BYTE    ch;
    UBYTE    index;
    BYTE    valid;

    while(TRUE)
    {
        TxString(serialdevice, channel,"Enter Route\r");
        showcommands(routes, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        valid = -1;
        index = 0;

        while(valid < 0)
        {
            while((ch = RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel,ch));
                /* store character in command string */
                routesetting[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(routesetting))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back
                    one character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to
                    space character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*
                    decrement string pointer so that character
                    that has been deleted is overwritten by
                    next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                routesetting[index] = 0;
                /* validate entry */
                valid = validate(routes, routesetting);
                if(valid<0)
                {
                    TxString(serialdevice, channel,"Invalid Route\r\n");
                    break;
                }
                else
                {
                    switch(valid)
                    {
					case 0:
						return BACTA;
					case 1:
						return RS232A;
					case 2:
						return RS232C;
					case 3:
						return RS232D;
					case 4:
						return CCTALK1;
					case 5:
						return CCTALK2;
					case 6:
						return TTL;
					case 7:
						return RS485;
                    }
                }
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... validate()
Author....... Marc Adams
Date......... 03/09/03
Description.. Validate the configuration setting entered for the
............. given serial port
Parameters... 1) Address of list of valid entries for the setting
............. 2) Address of the setting data entered by the user
Return....... Return the index into the array of valid settings
............. if the user enrty is valid
............. Return -1 if the entry is invalid
History...... 03/09/03 - written to test driver updates  - MSA
			  10/08/05 - fixed problems with warnings - JC
-------------------------------------------------------------------*/

static BYTE validate(const BYTE*const* settings, BYTE* entry)
{
    UBYTE  index;

    /* loop through valid setting */
    for(index=0; settings[index]; index++)
    {
        if((strcmp(settings[index],entry))==0)
        {
            return (index);
        }
    }

    return -1;
}


/********************************************************************
*** AUDIO
********************************************************************/
/*-------------------------------------------------------------------
Name......... demoaudio()
Author....... Marc Adams
Date......... 25/07/03
Description.. Demonstrate the operaton of the audio driver
Parameters... 1) Commands for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
			  10/08/05 - Added default to switch statement - JC
			  15/11/05 - Demo sound changed to use HSF file
-------------------------------------------------------------------*/

static void demoaudio(const BYTE*const* cmd ,
                      const DEVICE* serialdevice,BYTE channel)
{
    CMD command;

    /* initialise compact flash card */
    initcompactflash(serialdevice, channel);

    /* Only continue if flash card is mounted */
    if ( FAT32GetStatus( &ATADevice ) == MOUNTED )
    {
        /*
        change to audio directory
        */
        ChangeDir(&ATADevice, "audio");
    }
    else
    {
        TxString( serialdevice, channel, " Compact Flash not mounted\r" );
        return;
    }


    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {

        /* List files on compact flash card */
        case LISTFILESCMD:
            compactflashdir(serialdevice, channel);
            break;

        /* Set the channel volume */
        case SETVOLCMD:
            setaudiovolume(serialdevice,channel);
            break;

        /* Play a .HSF file */
        case PLAYCMD:
            playsample(serialdevice, channel, 0);
            break;

        /* Loop a .HSF file */
        case LOOPCMD:
            playsample(serialdevice, channel, 1);
            break;

        /* Silence a channel */
        case STOPCMD:
            stopchannel(serialdevice, channel);
            break;

        default:
            /* change directory back to root */
            ChangeDir(&ATADevice, "\\");
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... InitCompactFlash()
Author....... Mike Ballinger
Date......... 06/11/03
Description.. Initialise Compact Flash and display info
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 06/11/03 - written to test driver updates
			  10/08/05 - fixed compiler warnings
-------------------------------------------------------------------*/

static void initcompactflash( const DEVICE* serialdevice, BYTE channel )
{
    BYTE str[256];

    TxString( serialdevice, channel, "\n\rIdentify Drive: " );
    if ( ReadDriveID( &ATADevice ) != 0)
    {
        TxString( serialdevice, channel, "Timeout... \r" );
    }
    else
    {
        TxString( serialdevice, channel, "Pass...\r" );
    }

    sprintf( str, "\rModel: %s\rRev: %s\rSerial No: %s\rCylinders: %3d, Heads: %3d, Sectors: %3d\r\r",
        GetDriveModel( &ATADevice ),
        GetDriveRev( &ATADevice ),
        GetDriveSerialno( &ATADevice ),
        GetDriveCyls( &ATADevice ),
        GetDriveHeads( &ATADevice ),
        GetDriveSectors( &ATADevice ) );

    TxString( serialdevice, channel, str );

    TxString( serialdevice, channel, "Mounting Drive..." );
    if ( MountFAT32( &ATADevice ) != 0 )
    {
        TxString( serialdevice, channel, "Failed to mount drive\r\n" );
    }
    else
    {
        TxString( serialdevice, channel,
                 "FAT32 Partition found on Primary Partition\r" );
        sprintf( str, "\rVolume: %s  Size: %ld MBytes.\r\n",
                FAT32VolumeLabel( &ATADevice ),
                FAT32PartitionSize( &ATADevice ) );
        TxString( serialdevice, channel, str );
    }
}

/*-------------------------------------------------------------------
Name......... CompactFlashDir()
Author....... Mike Ballinger
Date......... 06/11/03
Description.. Prints directory listing on flash card
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 06/11/03 - written to test driver updates
-------------------------------------------------------------------*/

static void compactflashdir( const DEVICE* serialdevice, BYTE channel )
{
    BYTE str[256];
    DIR_ENTRY entry;

    /* initialise current directory structure and directory record pointer */
    ChangeDir(&ATADevice, ".");

    /* Only continue if flash card is mounted */
    if ( FAT32GetStatus( &ATADevice ) == MOUNTED )
    {
        /* get directory record */
        while ( NextDirRecord( &ATADevice, &entry ) )
        {
            sprintf( str, "%8s %-12s %10ld bytes %s\r",
                        DirEntryAttrib( &ATADevice, &entry ),
                        DirEntryAlias( &ATADevice, &entry ),
                        DirEntryFilesize( &ATADevice, &entry ),
                        DirEntryLongfilename( &ATADevice, &entry ) );
            TxString( serialdevice, channel, str );
        }
    }
    else
    {
        TxString( serialdevice, channel, "Flash card not yet mounted\r" );
    }
}

/*-------------------------------------------------------------------
Name......... setaudiovolume()
Author....... Mike Ballinger
Date......... 25/11/03
Description.. Demonstrate the operaton of the audio driver volume
............. control
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 25/11/03 - written to test driver updates
			  10/08/05 - Fixed problems with warnings- JC
-------------------------------------------------------------------*/

static void setaudiovolume(const DEVICE* serialdevice,BYTE channel)
{
    BYTE	str[255];
    BYTE	ch;
    LONG	index;
    BYTE	audiochannel;
    //LONG	volume;


    while(TRUE)
    {
        TxString( serialdevice, channel, "Enter audio channel:\r" );
        showcommands(volumecommands, serialdevice, channel);
        TxString(serialdevice, channel,"\r>");

        audiochannel = -1;
        index = 0;

        while(audiochannel < 0)
        {
            while((ch=RxChar(serialdevice, channel))<0);

            if((ch != '\r') && (ch!= '\b'))
            {
                /* Echo character */
                while(!TxChar(serialdevice, channel,ch));
                /* store character in command string */
                str[index++] = ch;
                /* Exceeded buffer size so start again */
                if(index == sizeof(str))
                {
                    break;
                }
            }
            else if(ch == '\b')
            {
                /* check for start of string */
                if(index>0 )
                {
                    /*
                    send backspace to move cursor back one
                    character console
                    */
                    while(!TxChar(serialdevice,channel,ch));
                    /*
                    overwrite character at current cursor
                    position with a space
                    */
                    while(!TxChar(serialdevice,channel,' '));
                    /*
                    send backspace to move cursor back to
                    space character on console
                    */
                    while(!TxChar(serialdevice,channel,ch));

                    /*
                    decrement string pointer so that character
                    that has been deleted is overwritten by
                    next character entered
                    */
                    index--;
                }
            }
            else
            {
                /* Echo character */
                while(!TxChar(serialdevice,channel,ch));
                /* Terminate string */
                str[index] = 0;
                /* validate entry */
                audiochannel = validate(volumecommands, str);
                if(audiochannel<0)
                {
                    TxString(serialdevice, channel,"Invalid channel\r\n");
                    index = 0;
                    TxString( serialdevice, channel, "Enter audio channel:\r" );
                    showcommands(volumecommands, serialdevice, channel);
                    TxString(serialdevice, channel,"\r>");
                }
            }
        }

        while(TRUE)
        {
            TxString(serialdevice, channel, "Please volume (0-255):\r" );
            getuserinput(serialdevice, channel, str, sizeof(str));
            volume = atoi(str);

            if ( ( volume < 0 ) || ( volume > 255 ) )
            {
                TxString(serialdevice, channel, "Invalid volume\r" );
            }
            else
            {
                /* Set volume */
                SetHSFChannelVolume( audiochannel , (UBYTE)volume );
                return;
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... playsample()
Author....... Marc Adams
Date......... 19/01/04
Description.. Play an audio sample over a given channel
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
............. 3) Loop flag: 0 - sample will be played once
.............               1 - sample will looped until stopped
Return....... None
History...... 19/01/04 - written to test driver updates
............. 15/11/05 - Rewritten to use HSF sound format - AD
-------------------------------------------------------------------*/

static void playsample( const DEVICE* serialdevice, BYTE channel,
                        BYTE playlooped )
{
    BYTE retcode;
    BYTE audiochannel;
    BYTE str[256];
    LONG index;
    BYTE ch;

    /* get the .hsf file name */
    TxString( serialdevice, channel, "Please enter HSF filename:\r" );
    getuserinput( serialdevice, channel, str, sizeof( str ) );

    /* initialise the .HSF file */
    retcode = OpenHSFFile(str, &HSFFile);

    if ( retcode != HSF_SUCCESS )
    {
        sprintf( str, "HSF analysis failure code = %d\r", retcode );
        TxString( serialdevice, channel, str );
        return;
    }


    TxString( serialdevice, channel, "Enter audio channel:\r" );
    showcommands(channelcommands, serialdevice, channel);
    TxString(serialdevice, channel,"\r>");

    audiochannel = -1;
    index = 0;

    while(audiochannel < 0)
    {
        while((ch=RxChar(serialdevice, channel))<0);

        if((ch != '\r') && (ch!= '\b'))
        {
            /* Echo character */
            while(!TxChar(serialdevice, channel,ch));
            /* store character in command string */
            str[index++] = ch;
            /* Exceeded buffer size so start again */
            if(index == sizeof(str))
            {
                break;
            }
        }
        else if(ch == '\b')
        {
            /* check for start of string */
            if(index>0 )
            {
                /*
                send backspace to move cursor back one
                character console
                */
                while(!TxChar(serialdevice,channel,ch));
                /*
                overwrite character at current cursor
                position with a space
                */
                while(!TxChar(serialdevice,channel,' '));
                /*
                send backspace to move cursor back to
                space character on console
                */
                while(!TxChar(serialdevice,channel,ch));

                /*
                decrement string pointer so that character
                that has been deleted is overwritten by
                next character entered
                */
                index--;
            }
        }
        else
        {
            /* Echo character */
            while(!TxChar(serialdevice,channel,ch));
            /* Terminate string */
            str[index] = 0;
            /* validate entry */
            audiochannel = validate(channelcommands, str);
            if(audiochannel<0)
            {
                TxString(serialdevice, channel,"Invalid channel\r\n");
                index = 0;
                TxString( serialdevice, channel, "Enter audio channel:\r" );
                showcommands(channelcommands, serialdevice, channel);
                TxString(serialdevice, channel,"\r>");
            }
        }
    }

    /*
    Ensure channel is only either AUDIO_LEFT0,
    AUDIO_RIGHT0 or AUDIO_STEREO0
    */
    if ( audiochannel >= AUDIO_LEFT1 )
    {
        SetHSFChannelVolume(( audiochannel - AUDIO_LEFT1 ), volume);
    }
    else
    {
        SetHSFChannelVolume( audiochannel, volume );
    }

    /* check whether sample is to be looped */
    if ( playlooped == 1 )
    {
        audiochannel = audiochannel | AUDIO_LOOPED;
    }

    PlayHSFFile(audiochannel, &HSFFile);
}

/*-------------------------------------------------------------------
Name......... stopchannel()
Author....... MArc Adams
Date......... 19/01/04
Description.. Stops any currently playing streamed audio on the
............. specified channel
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 19/01/04 - written to test driver updates
............. 15/11/05 - Rewritten to use HSF sound format - AD
-------------------------------------------------------------------*/

static void stopchannel( const DEVICE* serialdevice, BYTE channel )
{
    BYTE audiochannel;
    LONG index;
    BYTE ch;
    BYTE str[256];

    TxString( serialdevice, channel, "Enter audio channel to stop:\r" );
    showcommands(channelcommands, serialdevice, channel);
    TxString(serialdevice, channel,"\r>");

    audiochannel = -1;
    index = 0;

    while(audiochannel < 0)
    {
        while((ch=RxChar(serialdevice, channel))<0);

        if((ch != '\r') && (ch!= '\b'))
        {
            /* Echo character */
            while(!TxChar(serialdevice, channel,ch));
            /* store character in command string */
            str[index++] = ch;
            /* Exceeded buffer size so start again */
            if(index == sizeof(str))
            {
                break;
            }
        }
        else if(ch == '\b')
        {
            /* check for start of string */
            if(index>0 )
            {
                /*
                send backspace to move cursor back one
                character console
                */
                while(!TxChar(serialdevice,channel,ch));
                /*
                overwrite character at current cursor
                position with a space
                */
                while(!TxChar(serialdevice,channel,' '));
                /*
                send backspace to move cursor back to
                space character on console
                */
                while(!TxChar(serialdevice,channel,ch));

                /*decrement string pointer so that
                character that has been deleted is overwritten
                by next character entered
                */
                index--;
            }
        }
        else
        {
            /* Echo character */
            while(!TxChar(serialdevice,channel,ch));
            /* Terminate string */
            str[index] = 0;
            /* validate entry */
            audiochannel = validate(channelcommands, str);
            if(audiochannel<0)
            {
                TxString(serialdevice, channel,"Invalid channel\r\n");
                index = 0;
                TxString( serialdevice, channel,
                          "Enter audio channelto stop:\r" );
                showcommands(channelcommands, serialdevice, channel);
                TxString(serialdevice, channel,"\r>");
            }
        }
    }

    /* silence the channel */
    StopHSFChannel( audiochannel );
}


/*-------------------------------------------------------------------
Name......... getuserinput()
Author....... Marc Adams
Date......... 19/01/04
Description.. Get typed response from user.
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
............. 3) Data buffer to store user input
............. 4) Maximum buffer size
Return....... None.
History...... 19/01/04 - written to test driver updates
-------------------------------------------------------------------*/

static void getuserinput(const DEVICE* serialdevice, BYTE channel,
                         BYTE *returnbuffer, WORD maxBufferSize )
{
    LONG index;
    BYTE ch;

    TxString(serialdevice, channel,"\r>");

    index = 0;

    while(TRUE)
    {
        while((ch = RxChar(serialdevice, channel))<0);

        if((ch != '\r') && (ch!= '\b'))
        {
            /* Echo character */
            while(!TxChar(serialdevice, channel,ch));
            /* store character in command string */
            returnbuffer[index++] = ch;
            /* Exceeded buffer size so start again */
            if(index == maxBufferSize)
            {
                break;
            }
        }
        else if(ch == '\b')
        {
            /* check for start of string */
            if(index>0 )
            {
                /*
                send backspace to move cursor back one
                character console
                */
                while(!TxChar(serialdevice,channel,ch));
                /*
                overwrite character at current cursor
                position with a space
                */
                while(!TxChar(serialdevice,channel,' '));
                /*
                send backspace to move cursor back to
                space character on console
                */
                while(!TxChar(serialdevice,channel,ch));

                /*
                decrement string pointer so that character
                that has been deleted is overwritten by
                next character entered
                */
                index--;
            }
        }
        else
        {
            /* Echo character */
            while(!TxChar(serialdevice,channel,ch));
            /* Terminate string */
            returnbuffer[index] = 0;
            return;
        }
    }
}

/********************************************************************
*** VFD
********************************************************************/

/*-------------------------------------------------------------------
Name......... demovfd()
Author....... Marc Adams
Date......... 25/07/03
Description.. Demonstrate the operaton of VFD 1
Parameters... 1) Commands for the menu section(not used)
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void demovfd(const BYTE*const* cmd ,const DEVICE* serialdevice,
                    BYTE channel)
{
    TIMER vfdtimer;
    LONG index;
    const BYTE vfd_string1[] = {'T','E','S','T','I','N','G',' ','V','F',
                                'D',' ','1',' ','O','N',' ','P',
                                'L','U','T','O',' ','6',' ','\0'};

    ResetVFD1();

    /* Force VFD update */
    vfdtimer = 0;

    while(TRUE)
    {
        if(RxChar(serialdevice, channel)>=0)
        {
            ResetVFD1();
            for(index=0;index<16;index++) TxVFD1(' ');
            /* Kill timer */
            SetTimer(&vfdtimer,0);
            /* reset string counters */
            output_string_pos = 0;
            input_string_pos = 0;
            /* Exit */
            return;
        }
        else
        {
            if(vfdtimer == 0)
            {
                scrollvfd1text(vfd_string1);
                SetTimer(&vfdtimer,15);
            }

        }

    }
}

/*-------------------------------------------------------------------
Name......... scrollvfdtext()
Author....... Marc Adams
Date......... 25/07/03
Description.. Scrolls a null terminated text string on vfd1
............. Text string is displayed with an offset of 1 every
............. time the function is called
Parameters... 1) Address of text string to scroll
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/


static void scrollvfd1text(const BYTE *text)
{
    BYTE temp_vfd_string[16];
    BYTE inputlength;

    inputlength = strlen(text);

    output_string_pos = 0;

    while(output_string_pos < sizeof(temp_vfd_string))
    {
        if(input_string_pos == inputlength)
        {
            input_string_pos = 0;
        }

        temp_vfd_string[output_string_pos] = text[input_string_pos];

        output_string_pos++;
        input_string_pos++;
    }

    input_string_pos -= 15;

    if(input_string_pos < 0)
    {
        input_string_pos += inputlength;
    }

    for(output_string_pos = 0; output_string_pos < sizeof(temp_vfd_string);
        output_string_pos++)
    {
        TxVFD1(temp_vfd_string[output_string_pos]);
    }
}


/********************************************************************
*** PIC18 CSD
********************************************************************/

/*-------------------------------------------------------------------
Name......... democsd()
Author....... Marc Adams
Date......... 25/07/03
Description.. Demonstrate the operaton of the PIC18 CSD
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static void democsd(const BYTE*const* cmd ,
                    const DEVICE* serialdevice,BYTE channel)
{

    LONG    index;
    P18TIME     picrtctime;
    TESTPICCMD  command;
    BYTE    str[6];
    BYTE    ch;
    TIMER   clockupdatetimer;
    BYTE    logindex;
    BYTE    pwroffinput;
    BYTE    input;
    BYTE    percentdata;
    BYTE    stakeprizedata;
    BYTE    dil1data;
    BYTE    dil2data;
    BYTE    revisiondata;
    BYTE    dil_data_mask;
    BYTE	return_code;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        case READRTCCMD:
            SetTimer(&clockupdatetimer,100);

            while((ch=RxChar(serialdevice,channel)) < 0)
            {
                if(clockupdatetimer==0)
                {
                    GetPIC18Time(&PICDevice, &picrtctime);
                    TxString(serialdevice,channel,"\r");
                    sprintf(str, "%d", picrtctime.date);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.month);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.year);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel,"\r");
                    sprintf(str, "%d", picrtctime.hours);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.minutes);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.seconds);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel,"\r");
                    TxString(serialdevice,channel,"\rPress any key to continue\r");
                    SetTimer(&clockupdatetimer,100);
                }
            }
            /* Kill timer */
            SetTimer(&clockupdatetimer,0);
            /* Exit */
            break;

        case READRTCFINECMD:
            SetTimer(&clockupdatetimer,5);

            while((ch=RxChar(serialdevice,channel)) < 0)
            {
                if(clockupdatetimer==0)
                {
                    GetPIC18TimeFine(&PICDevice, &picrtctime);
                    TxString(serialdevice,channel,"\r");
                    sprintf(str, "%d", picrtctime.date);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.month);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.year);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel,"\r");
                    sprintf(str, "%d", picrtctime.hours);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.minutes);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.seconds);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel," : ");
                    sprintf(str, "%d", picrtctime.hundredths);
                    TxString(serialdevice,channel,str);
                    TxString(serialdevice,channel,"\r");
                    TxString(serialdevice,channel,"\rPress any key to continue\r");
                    SetTimer(&clockupdatetimer,5);
                }
            }
            /* Kill timer */
            SetTimer(&clockupdatetimer,0);
            /* Exit */
            break;

        case WRITERTCCMD:
            gettime(serialdevice, channel, &picrtctime);
            SetPIC18Time(&PICDevice, &picrtctime);
            break;

        case READLOGCMD:
            logindex = getindex(serialdevice, channel, &input);

            TxString(serialdevice,channel,"\rLog Index ");
            while(!TxChar(serialdevice,channel,input));
            TxString(serialdevice,channel,"\r");

            return_code = GetPIC18Log(&PICDevice, logindex, &picrtctime, &pwroffinput);

            switch(return_code)
            {
			case TRUE:
				TxString(serialdevice,channel,"Date : ");
				sprintf(str, "%d", picrtctime.date);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel," / ");
				sprintf(str, "%d", picrtctime.month);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel," / ");
				sprintf(str, "%d", picrtctime.year);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel,"\r");
				TxString(serialdevice,channel,"Time : ");
				sprintf(str, "%d", picrtctime.hours);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel," : ");
				sprintf(str, "%d", picrtctime.minutes);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel," : ");
				sprintf(str, "%d", picrtctime.seconds);
				TxString(serialdevice,channel,str);
				TxString(serialdevice,channel,"\r");
				TxString(serialdevice,channel,"Switch status :\r");
				if(pwroffinput & 0x01)
				{
					TxString(serialdevice,channel,"Switch 1 ON\r");
				}
				else
				{
					TxString(serialdevice,channel,"Switch 1 OFF\r");
				}

				if(pwroffinput & 0x02)
				{
					TxString(serialdevice,channel,"Switch 2 ON\r");
				}
				else
				{
					TxString(serialdevice,channel,"Switch 2 OFF\r");
				}

				if(pwroffinput & 0x04)
				{
					TxString(serialdevice,channel,"Switch 3 ON\r");
				}
				else
				{
					TxString(serialdevice,channel,"Switch 3 OFF\r");
				}

				if(pwroffinput & 0x08)
				{
					TxString(serialdevice,channel,"Switch 4 ON\r");
				}
				else
				{
					TxString(serialdevice,channel,"Switch 4 OFF\r");
				}
				break;
			case ERR_MBUS_FAIL:
				TxString(serialdevice,channel,"problem communication with pic try again\r");
				break;
			case ERR_RECORD_EMPTY:
				sprintf(str, "no entry for %d \r ", logindex);
				TxString(serialdevice,channel,str);
				break;
			case ERR_WRONG_INDEX:
				TxString(serialdevice,channel,"wrong index! \r");
				break;
            }
            TxString(serialdevice,channel,"\r\r");
            break;

        case CLEARLOGCMD:
            ClearPIC18Log(&PICDevice);
            TxString(serialdevice,channel,"Switch Log cleared\r\r");
            break;

        case READPERCENTCMD:

            GetPIC18Percent(&PICDevice, &percentdata);
            TxString(serialdevice,channel,"Percentage Key data\r\r");
            if(percentdata & 0x01)
            {
                TxString(serialdevice,channel,"x 3 ");
            }

            if(percentdata & 0x02)
            {
                TxString(serialdevice,channel,"x 2 ");
            }

            if(percentdata & 0x04)
            {
                TxString(serialdevice,channel,"x 1 ");
            }

            if(percentdata & 0x08)
            {
                TxString(serialdevice,channel,"x 0 ");
            }

            TxString(serialdevice,channel,"\r\r");

            break;

        case READSTAKECMD:
            GetPIC18Stake(&PICDevice, &stakeprizedata);
            TxString(serialdevice,channel,"Stake/Prize Key data\r\r");
            if(stakeprizedata & 0x01)
            {
                TxString(serialdevice,channel,"ST0 ");
            }
            if(stakeprizedata & 0x02)
            {
                TxString(serialdevice,channel,"ST1 ");
            }

            if(stakeprizedata & 0x04)
            {
                TxString(serialdevice,channel,"ST2 ");
            }

            if(stakeprizedata & 0x08)
            {
                TxString(serialdevice,channel,"JK0 ");
            }

            if(stakeprizedata & 0x10)
            {
                TxString(serialdevice,channel,"JK1 ");
            }

            if(stakeprizedata & 0x20)
            {
                TxString(serialdevice,channel,"JK2 ");
            }

            if(stakeprizedata & 0x40)
            {
                TxString(serialdevice,channel,"JK3 ");
            }

            TxString(serialdevice,channel,"\r\r ");
            break;

        case READDIL1CMD:
            GetPIC18Dil1(&PICDevice, &dil1data);
            TxString(serialdevice,channel,"DIL Switch 1 reading\r\r");

            dil_data_mask = 1;

            for(index=0; index<DIL_DATA_SIZE; index++)
            {
                if(dil1data & dil_data_mask)
                {
                    TxString(serialdevice,channel,"1 ");
                }
                else
                {
                    TxString(serialdevice,channel,"0 ");
                }
                if(dil_data_mask == 1)
                {
                    dil_data_mask++;
                }
                else
                {
                    dil_data_mask *= 2;
                }
            }
            TxString(serialdevice,channel,"\r\r ");
            break;

        case READDIL2CMD:
            GetPIC18Dil2(&PICDevice, &dil2data);
            TxString(serialdevice,channel,"DIL Switch 2 reading\r\r");

            dil_data_mask = 1;

            for(index=0; index<DIL_DATA_SIZE; index++)
            {
                if(dil2data & dil_data_mask)
                {
                    TxString(serialdevice,channel,"1 ");
                }
                else
                {
                    TxString(serialdevice,channel,"0 ");
                }

                if(dil_data_mask == 1)
                {
                    dil_data_mask++;
                }
                else
                {
                    dil_data_mask *= 2;
                }
            }
            TxString(serialdevice,channel,"\r\r ");
            break;

		case READREVISION:
			GetPIC18Revision(&PICDevice, &revisiondata);
			TxString(serialdevice,channel,"Revision Number> ");
			sprintf(str, "%d", revisiondata);
			TxString(serialdevice,channel,str);
			TxString(serialdevice,channel,"\r\n");
			break;


        case EXITCSDCMD:
            return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... gettime()
Author....... Marc Adams
Date......... 25/07/03
Description.. Function to allow the user to populate the real time
............. structure used to set the PIC real time clock
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
............. 3) time structure to be populated
Return....... None
History...... 25/07/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed problems with warnings - JC
-------------------------------------------------------------------*/

static void gettime(const DEVICE* serialdev,BYTE channel, P18TIME* time)
{
    BYTE    entry;
    WORD    setting;


	entry = YEAR;

    time->seconds = 0;
    time->hundredths = 0;

    while(TRUE)
    {
        switch(entry)
        {
		case YEAR:
			TxString(serialdev,channel,"\rEnter Year\r");
			setting = getinput(serialdev, channel, &entry);
			if(setting > 0)
			{
				time->year = setting;
			}
		break;

		case MONTH:
			TxString(serialdev,channel,"\rEnter Month\r");
			setting = getinput(serialdev, channel, &entry);
			if(setting > 0)
			{
				time->month = setting;
			}
		break;

		case DAY:
			TxString(serialdev,channel,"\rEnter Date\r");
			setting = getinput(serialdev, channel, &entry);
			if(setting > 0)
			{
				time->date = setting;
			}
		break;

		case HOURS:
			TxString(serialdev,channel,"\rEnter Hours\r");
			setting = getinput(serialdev, channel, &entry);
			if(setting >= 0)
			{
				time->hours = setting;
			}
		break;

		case MINS:
			TxString(serialdev,channel,"\rEnter Minutes\r");
			setting = getinput(serialdev, channel, &entry);
			if(setting >= 0)
			{
				time->minutes = setting;
			}

		break;

		default:
		return;
        }
    }
}

/*-------------------------------------------------------------------
Name......... getinput()
Author....... Marc Adams
Date......... 25/07/03
Description.. Function used to get and validate the input entered
............. by the user to populate the time structure used to
............. set the CSD real time clock
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
............. 3) Address of the entry to the time structure
Return....... Validated user input or
			  -1 on failure
History...... 25/07/03 - written to test driver updates  - MSA
			  10/08/05 - Fixed compiler warning - JC
-------------------------------------------------------------------*/

static WORD getinput(const DEVICE* serialdev,BYTE channel, BYTE *entry)
{
    BYTE ch;
    LONG index;
    WORD setting;
    BYTE str[6];

    index=0;
    TxString(serialdev,channel,"\r>");

    while(TRUE)
    {
        /* get character */
        while((ch=RxChar(serialdev,channel))<0);
        /* Echo character */
        while(!TxChar(serialdev,channel,ch));

        if(ch!='\r')
        {
            str[index++]=ch;        /* Get next char of command */
            if(index==sizeof(str))
            {
   									/* Exceeded buffer size so start again */
                return -1;
            }
        }
        else
        {
            str[index]=0;

            index = 0;
            switch(*entry)
            {
			case YEAR:
				setting = atoi(&str[index]);
				if((setting < 2000) || (setting > 2500))
				{
					return (-1);
				}
				else
				{
					*entry+=1;
					return (setting);
				}
			break;
			case MONTH:
				setting = atoi(&str[index]);
				if((setting < 1) || (setting > 12))
				{
					return (-1);
				}
				else
				{
					*entry+=1;
					return (setting);
				}
			case DAY:
				setting = atoi(&str[index]);
				if((setting < 1) || (setting > 31))
				{
					return (-1);
				}
				else
				{
					*entry+=1;
					return (setting);
				}

			case HOURS:
				setting = atoi(&str[index]);
				if((setting < 0) || (setting > 23))
				{
					return (-1);
				}
				else
				{
					*entry+=1;
					return (setting);
				}

			case MINS:
				setting = atoi(&str[index]);
				if((setting < 0) || (setting > 59))
				{
					return (-1);
				}
				else
				{
					*entry+=1;
					return (setting);
				}
            }
        }
    }
}

/*-------------------------------------------------------------------
Name......... getindex()
Author....... Marc Adams
Date......... 25/07/03
Description.. Function to allow the user to specify the security switch
............. index to be read
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
............. 3) Address of the byte used to store user input
Return....... Switch log index value to read
History...... 25/07/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static BYTE getindex(const DEVICE* serialdev, BYTE channel, BYTE *ch)
{
    LONG index;

	TxString(serialdev,channel,"\rEnter Log Index 1-10\r");
    TxString(serialdev,channel,"\r>");

    /* get character */
    while((*ch=RxChar(serialdev,channel))<0);
    /* Echo character */
    while(!TxChar(serialdev,channel,*ch));

    index = atoi(ch);

	return (index);

}


/********************************************************************
*** E2PROM
********************************************************************/

/*-------------------------------------------------------------------
Name......... demoe2rom()
Author....... Ed Langley
Date......... 1/10/2004
Description.. Demonstrate the operaton of the EEPROM
Parameters... 1) Commands  for the menu section
............. 2) Address of serial ouput device connected to console
............. 3) Serial device output channel
Return....... None
History...... 1/10/2004 - written for the next dev kit release - EL
-------------------------------------------------------------------*/

static void demoe2rom(const BYTE*const* cmd ,
                    const DEVICE* serialdevice,BYTE channel)
{
    TESTE2ROMCMD  command;
	WORD	offset;
	WORD	numbytes;
	BYTE	rwbuffer[E2ROM_BUFFER_SIZE];
	BYTE	str[20];
	BYTE	output[10];
	WORD	index, endline;
	BYTE*	asciiptr;
	UBYTE*	hexptr;

    while(TRUE)
    {
        command=getcommand(cmd,serialdevice,channel);
        switch(command)
        {
        case READE2ROMCMD:
			/* allow user to read a range of their choice from the eeprom */
			TxString(serialdevice, channel, "\rRead from E2ROM" );
			TxString(serialdevice, channel, "\rPlease enter offset:" );
			getuserinput(serialdevice, channel, str, sizeof(str));
			offset = atoi(str);
			TxString(serialdevice, channel, "\rPlease enter number of bytes to read:" );
			getuserinput(serialdevice, channel, str, sizeof(str));
			numbytes = atoi(str);

			switch(ReadE2Rom(&E2RomDevice, offset, numbytes, rwbuffer))
			{
			case E2_OUT_OF_BOUNDS:
				TxString(serialdevice, channel, "choose correct range for eeprom\r");
				break;
			case E2_READ_FAILED:
				TxString(serialdevice, channel, "read failed\r");
				break;
			default:
				asciiptr = rwbuffer;
				hexptr = rwbuffer;
				endline = 26; /* number of hex bytes per line */

				/* print a listing in hex */
				for(index = 1; index<=numbytes; index++)
				{
					sprintf(str, "%2X ", *hexptr);
					TxString(serialdevice, channel, str);
					hexptr++;

					if(index >= endline)
					{
						TxString(serialdevice, channel, "\r");
						endline += 26;
					}
				}

				TxString(serialdevice, channel, "\r\r");
				endline = 50; /* number of ascii chars per line */

				/* and also in ascii */
				for(index = 1; index<=numbytes; index++)
				{
					/* only print printable chars */
					if( (*asciiptr >= 32) && (*asciiptr <= 126) )
					{
						sprintf(str, "%c", *asciiptr);
					}
					else
					{
						sprintf(str, ".");
					}

					TxString(serialdevice, channel, str);
					asciiptr++;

					if(index >= endline)
					{
						TxString(serialdevice, channel, "\r");
						endline += 50;
					}
				}

				/* clear buffers */
				for(index = 0; index<100; index++)
				{
					output[index] = 0;
				}

				for(index = 0; index<E2ROM_BUFFER_SIZE;index++)
				{
					rwbuffer[index] = 255;
				}

				TxString(serialdevice, channel, "\r\r");
				break;
			}

        	break;
        case WRITEE2ROMCMD:
			/* allow user to write to their choice of range on the eeprom */
			TxString(serialdevice, channel, "\rWrite to E2ROM" );
			TxString(serialdevice, channel, "\rPlease enter offset:" );
			getuserinput(serialdevice, channel, str, sizeof(str));
			offset = atoi(str);

			TxString(serialdevice, channel, "\rPlease enter num of bytes to write:" );
			getuserinput(serialdevice, channel, str, sizeof(str));
			numbytes = (WORD)atoi(str);

			TxString(serialdevice, channel, "\rPlease enter some text to write:" );
			getuserinput(serialdevice, channel, rwbuffer, sizeof(rwbuffer));

			switch(WriteE2Rom(rwbuffer, numbytes, &E2RomDevice, offset))
			{
				case E2_OUT_OF_BOUNDS:
					TxString(serialdevice, channel, "choose correct range for eeprom\r\r");
					break;
				case E2_WRITE_FAILED:
					TxString(serialdevice, channel, "write failed\r\r");
					break;
				default:
					TxString(serialdevice, channel, "data written\r\r");
					break;
			}

			/* clear buffer */
			for(index = 0; index<512;index++)
			{
				rwbuffer[index] = 255;
			}
        	break;
        case EXITE2ROMCMD:
        	return;
		}
	}
}


/*-------------------------------------------------------------------
Name......... time_out_demo()
Author....... Marc Adams
Date......... 12/09/03
Description.. Demo function that runs after a specified time period
............. of user inactivity.  This function polls for a key
............. press from the user; if one has not occurred the auto
............. demo code is executed, if a key press is detected all
............. hardware used in the auto demo is cleared and normal
............. menu driven operation resumes
Parameters... 1) Address of serial ouput device connected to console
............. 2) Serial device output channel
Return....... None
History...... 12/09/03 - written to test driver updates  - MSA
-------------------------------------------------------------------*/

static void time_out_demo(const DEVICE* serialdev, BYTE channel)
{
    const BYTE vfd_string[] = {' ',' ',' ',' ','P','L','U','T',
                                'O',' ','6',' ',' ',' ',' ',' '};
    BYTE ch;
    LONG index;
    LONG index2;

    TxString(serialdev,channel,"\rPress any key to return to menu\r\r");

    /* configure serial ports for automated demo */

    /* Configure the Coldfire UART channel 1 for the automated demo */
    ConfigureSerial(&SerialDevice, 1, BAUD1200, BAUD1200, BITS8, STOP1,
                    PARITYNONE, TXNONE, RXNONE);

    /* Configure DUART channel 0 for the automated demo */
    ConfigureSerial(&DUART0Device, 0, BAUD1200, BAUD1200, BITS8, STOP1,
                    PARITYNONE, TXNONE, RXNONE);

    /* Configure DUART channel 1 for the automated demo */
    ConfigureSerial(&DUART0Device, 1, BAUD1200, BAUD1200, BITS8, STOP1,
                    PARITYNONE, TXNONE, RXNONE);

    /* Configure FPUART 0 for the automated demo */
    ConfigureSerial(&FPUART0Device, 0, BAUD1200, BAUD1200, BITS8, STOP1,
                    PARITYNONE, TXNONE, RXNONE);

    /* Configure FPUART 1 for the automated demo */
    ConfigureSerial(&FPUART1Device, 0, BAUD1200, BAUD1200, BITS8, STOP1,
                    PARITYNONE, TXNONE, RXNONE);

    /* Route Coldfire UART channel 1 */
    SetRouting( &SerialDevice, 1, RS232A );

    /* Route DUART channel 0 */
    SetRouting( &DUART0Device, 0, RS232C);

    /* Route DUART channel 0 */
    SetRouting( &DUART0Device, 1, RS232D );

    /* Route FPUART 0 */
    SetRouting( &FPUART0Device, 0, CCTALK1 );

    /* Route FPUART 1 */
    SetRouting( &FPUART1Device, 0, CCTALK2 );

    ch = RxChar( serialdev, channel );

    while( ch < 0)
    {
        /* run automated demo code */
        autodemo( );

        /* check for key stroke to re-enter menu code */
        ch = RxChar( serialdev, channel );
    }


    /* Clear VFD */
    ResetVFD1();
    for(index=0;index<16;index++)
    {
        TxVFD1(vfd_string[index]);
    }

    /* Clear open drain outputs */
    for(index = 0;index < 64;index++)
    {
            OutputOff(outputtable[index]);
    }

    /* Clear LEDS */
    for(index = 0;index < 16;index ++)
    {
        LEDBits(digit_table0_15[index],255,OFF);
        LEDBits(digit_table16_31[index],255,OFF);
    }

    /* Clear LAMPS */
    for ( index = 0; index < 16; index ++ )
    {
        for ( index2 = 0; index2 < 16; index2 ++ )
        {
            Lamp( LAMP( index, index2 ), OFF );
        }
    }

    /* reset global counters */
    mpx_update = 0;
    previous_od = 16;
    current_od = 16;
    current = 0;
    brightness = 0;
    bright_index = 0;
    animationMode = 0;

}

/*-------------------------------------------------------------------
Name......... LampEdge()
Author....... Mike Ballinger
Date......... 13/01/04
Description.. Turns on lamps with address relative to edge
Parameters... 1) edge: 0 (outer) to 7 (inner)
............. 2) position: 0-3  (edge 7)
.............             0-11 (edge 6)
.............             0-19 (edge 5)
.............             0-27 (edge 4)
.............             0-35 (edge 3)
.............             0-43 (edge 2)
.............             0-51 (edge 1)
.............             0-59 (edge 0)
............. 3) status - ON or OFF
Return....... None
History...... 13/01/04 - Initial version - MJB
-------------------------------------------------------------------*/

void LampEdge( BYTE edge, BYTE position, const STATUS status )
{
    BYTE side0End, side1End, side2End, side3End;

    side0End = 15 - ( 2 * edge );
    side1End = side0End + 15 - ( 2 * edge );
    side2End = side1End + 15 - ( 2 * edge );
    side3End = side2End + 15 - ( 2 * edge );

    if ( ( position >= 0 ) && ( position <= side0End ) )
    {
        Lamp( LAMP( ( position + edge ), edge ), status );
    }
    else if ( ( position > side0End ) && ( position <= side1End ) )
    {
        Lamp( LAMP( ( side0End + edge ), ( position - side0End + edge ) ), status );
    }
    else if ( ( position > side1End ) && ( position <= side2End ) )
    {
        Lamp( LAMP( ( side2End - position + edge ), ( side0End + edge ) ), status );
    }
    else if ( ( position > side2End ) && ( position <= side3End ) )
    {
        Lamp( LAMP( edge, ( side3End - position + edge ) ), status );
    }
}

/*-------------------------------------------------------------------
Name......... autodemo()
Author....... Mike Ballinger
Date......... 13/01/04
Description.. Automated demo function that checks the value of each
............. of the functionality counters. If the value of the
............. counter has reached 0 the demo code is executed
Parameters... None
Return....... None
History...... 13/01/04 - initial version - MJB
-------------------------------------------------------------------*/

static void autodemo()
{
    const  BYTE heber_vfd_string[] = {'*','*',' ','H','E','B','E','R',' ',
                                      ' ','L','T','D',' ','*','*'};
    const  BYTE pluto_vfd_string[] = {'*','*',' ',' ','P','L','U','T','O'
                                      ,' ',' ','6',' ',' ','*','*'};
    static STATUS st;
    static BYTE retries;
    static BYTE deleteLastDots;
    static BYTE wormPos[8];
    static BYTE drainOutputsOn = 1;
    static WORD od_idx = 0;
    static BYTE vfdDisplayMessage = 1;
    const  OUTPUT *od_ix;
    const  LAMP *lp_ix;
    LONG   index, index2, pos;
    BYTE   edgeFinish;

    if( lamps_update_count == 0 )
    {
        /* Update the Multiplexed Lamps */
        switch( mpx_update )
        {
		case 0:
			/* Diagonal fill initialisation state */
			current = 1;
			retries = 3;
			st = ON;
			mpx_update++;
			break;

		case 1:
			/*
			Diagonal lamps full brightness fill or empty
			(depending on st)
			*/
			lp_ix = linelst[current];

			SetDefaultBrightness( );

			if ( st == ON )
			{
				if ( linelst[current] != 0 )
				{
					while( *lp_ix != EOF_LAMPS )
					{
						Lamp( *lp_ix++, st );
					}
					current++;
				}
				else
				{
					current --;
					st = OFF;

					retries --;
					if ( retries == 0 ) mpx_update++;
				}
			}
			else if ( st == OFF )
			{
				if ( linelst[current] != 0 )
				{
					while( *lp_ix != EOF_LAMPS )
					{
						Lamp( *lp_ix++, st );
					}
					current--;
				}
				else
				{
					current ++;
					st = ON;
				}
			}
			break;

		case 2:
			/* Flash filled screen initialisation state */
			retries = 40;

			/* Clear LAMPS */
			for ( index = 0; index < 16; index ++ )
			{
				for ( index2 = 0; index2 < 16; index2 ++ )
				{
					Lamp( LAMP( index, index2 ), OFF );
				}
			}

			SetDefaultBrightness( );

			/* display flashing X */
			lp_ix=big_lamp_x;
			while(*lp_ix!=EOF_LAMPS)
			{
				Lamp(*lp_ix++,FF1);
			}

			mpx_update++;
			break;

		case 3:
			retries --;
			if ( retries == 0 )
			{
				/* Clear all lamps */
				for ( index = 0; index < 16; index ++ )
				{
					for ( index2 = 0; index2 < 16; index2 ++ )
					{
						Lamp( LAMP( index, index2 ), OFF );
					}
				}

				mpx_update++;
			}
			break;

		case 4:
			/* Crisscross initialisation state */
			current = 0;
			retries = 3;
			mpx_update++;
			break;

		case 5:
			/* Forward crisscross */
			for ( index = 0; index < 16; index ++ )
			{
				if ( current != 0 )
				{
					if ( current != index )
					{
						Lamp( LAMP( index, ( current - 1 ) ), OFF );
						Lamp( LAMP( ( current - 1 ), index ), OFF );
					}
				}

				Lamp( LAMP( index, current ), ON );
				Lamp( LAMP( current, index ), ON );
			}

			if ( current == 15 ) mpx_update ++;
			else current ++;
			break;

		case 6:
			/* Reverse crisscross */
			for ( index = 0; index < 16; index ++ )
			{
				if ( ( current - 1 ) != index )
				{
					Lamp( LAMP( index, current ), OFF );
					Lamp( LAMP( current, index ), OFF );
				}

				Lamp( LAMP( index, ( current - 1 ) ), ON );
				Lamp( LAMP( ( current - 1 ), index ), ON );
			}

			if ( current == 1 )
			{
				retries --;
				if ( retries == 0 )
				{
					mpx_update++;
				}
				else
				{
					mpx_update --;
				}
			}
			else current --;
			break;

		case 7:
			/* Crisscross initialisation state (with faded background) */
			current = 0;
			retries = 3;

			/* Fill in faded background */
			for ( index = 0; index < 16; index ++ )
			{
				for ( index2 = 0; index2 < 16; index2 ++ )
				{
					SetLampBrightness( LAMP( index, index2 ), ON, 0x10 );
					Lamp( LAMP( index, index2 ), ON );
				}
			}
			mpx_update++;
			break;

		case 8:
			/* Forward crisscross */
			for ( index = 0; index < 16; index ++ )
			{
				if ( current != 0 )
				{
					if ( current != index )
					{
						SetLampBrightness( LAMP( index, ( current - 1 ) ), ON, 0x10 );
						SetLampBrightness( LAMP( ( current - 1 ), index ), ON, 0x10 );
						Lamp( LAMP( index, ( current - 1 ) ), ON );
						Lamp( LAMP( ( current - 1 ), index ), ON );
					}
				}

				SetLampBrightness( LAMP( index, current ), ON, 0xff );
				SetLampBrightness( LAMP( current, index ), ON, 0xff );
				Lamp( LAMP( index, current ), ON );
				Lamp( LAMP( current, index ), ON );
			}

			if ( current == 15 )
			{
				mpx_update ++;
			}
			else
			{
				current ++;
			}
			break;

		case 9:
			/* Reverse crisscross */
			for ( index = 0; index < 16; index ++ )
			{
				if ( ( current - 1 ) != index )
				{
					SetLampBrightness( LAMP( index, current ), ON, 0x10 );
					SetLampBrightness( LAMP( current, index ), ON, 0x10 );
					Lamp( LAMP( index, current ), ON );
					Lamp( LAMP( current, index ), ON );
				}

				SetLampBrightness( LAMP( index, ( current - 1 ) ), ON, 0xff );
				SetLampBrightness( LAMP( ( current - 1 ), index ), ON, 0xff );
				Lamp( LAMP( index, ( current - 1 ) ), ON );
				Lamp( LAMP( ( current - 1 ), index ), ON );
			}

			if ( current == 1 )
			{
				retries --;
				if ( retries == 0 )
				{
					mpx_update++;
				}
				else
				{
					mpx_update --;
				}
			}
			else current --;
			break;

		case 10:
			/* Remove last crisscross pattern */
			SetDefaultBrightness( );
			for ( index = 0; index < 16; index ++ )
			{
				for ( index2 = 0; index2 < 16; index2 ++ )
				{
					Lamp( LAMP( index, index2 ), OFF );
				}
			}

			mpx_update ++;
			break;

		case 11:
			/* Chasing worms initialisation */
			deleteLastDots = 0;
			retries = 180;
			for ( index = 0; index < 8; index ++ )
			{
				wormPos[index] = 0;
			}
			mpx_update ++;
			break;

		case 12:
			if ( deleteLastDots == 0 )
			{
				/* Draw initial "worms" */
				for ( index = 0; index < 8; index ++ ) LampEdge( 0, (BYTE)index + wormPos[0], ON );
				for ( index = 0; index < 7; index ++ ) LampEdge( 1, (BYTE)index + wormPos[1], ON );
				for ( index = 0; index < 6; index ++ ) LampEdge( 2, (BYTE)index + wormPos[2], ON );
				for ( index = 0; index < 5; index ++ ) LampEdge( 3, (BYTE)index + wormPos[3], ON );
				for ( index = 0; index < 4; index ++ ) LampEdge( 4, (BYTE)index + wormPos[4], ON );
				for ( index = 0; index < 3; index ++ ) LampEdge( 5, (BYTE)index + wormPos[5], ON );
				for ( index = 0; index < 2; index ++ ) LampEdge( 6, (BYTE)index + wormPos[6], ON );
				for ( index = 0; index < 1; index ++ ) LampEdge( 7, (BYTE)index + wormPos[7], ON );
				deleteLastDots = 1;
			}
			else
			{
				for ( index = 0; index < 8; index ++ )
				{
					edgeFinish = 59 - ( index * 8 );
					LampEdge( index, wormPos[index], OFF );
					pos = wormPos[index] + 8 - index;
					if ( pos > edgeFinish )
					{
						pos = pos - edgeFinish - 1;
					}
					LampEdge( index, pos, ON );
					if ( wormPos[index] < edgeFinish )
					{
						wormPos[index] ++;
					}
					else
					{
						wormPos[index] = 0;
					}
				}
			}

			if ( retries ) retries --;
			else mpx_update ++;
			break;

		case 13:
			/* Remove worms */
			for ( index = LC0; index <= LC15; index ++ )
			{
				for ( index2 = LR0; index2 <= LR15; index2 ++ )
				{
					Lamp( LAMP( index, index2 ), OFF );
				}
			}
			mpx_update = 0;
			break;
        }
        lamps_update_count = LAMP_ANIM_TIME;
    }

    /* Update the VFD */
    if ( vfd_counter == 0 )
    {
        for( index = 0; index < 16; index++ )
        {
            if(vfdDisplayMessage)
            {
                TxVFD1( heber_vfd_string[index] );
            }
            else
            {
                TxVFD1( pluto_vfd_string[index] );
            }
        }

        vfdDisplayMessage = 1 - vfdDisplayMessage;
        vfd_counter = VFD_ANIM_TIME;
    }

    /* Send a character over the serial ports */
    if ( serial_counter == 0 )
    {
        TxString(&SerialDevice,1,"RS232 A");
        TxString(&DUART0Device,0,"RS232 C");
        TxString(&DUART0Device,1,"RS232 D");
        TxString(&FPUART0Device,0,"CCTALK 1");
        TxString(&FPUART1Device,0,"CCTALK 2");

        /* Clear recieve buffer in case loop back switches are ON */
        ClearRx( &SerialDevice, 1 );
        ClearRx( &DUART0Device, 0 );
        ClearRx( &DUART0Device, 1 );
        ClearRx( &FPUART0Device, 0 );
        ClearRx( &FPUART1Device, 0 );

        serial_counter = SERIAL_ANIM_TIME;
    }

    if ( drainOutputsCounter == 0 )
    {
        od_ix = od_outputs[od_idx];
        while (*od_ix != EOF_OUTPUTS)
        {
            if ( drainOutputsOn )
            {
				OutputOn(*od_ix++);
			}
            else
            {
				OutputOff(*od_ix++);
			}
        }

        od_idx++;
        if ( od_outputs[od_idx] == 0 )
        {
            od_idx = 0;
            drainOutputsOn = 1 - drainOutputsOn;
        }

        drainOutputsCounter = DRAIN_OUTPUT_TIME;
    }

    /* Update the LEDS */
    if(leds_update_counter == 0)
    {
        /* update both LED displays */
        UpdateLEDs();
        leds_update_counter = LED_ANIM_TIME;
    }


}

/*-------------------------------------------------------------------
Name......... UpdateLEDs()
Author....... Mike Ballinger
Date......... 09/01/04
Description.. Function to display RTC and perform simple LED demo
Parameters... None
Return....... None
History...... 09/01/04 - Initial version - MJB
			  10/08/05 - fixed problems with warnings - JC
-------------------------------------------------------------------*/

static void UpdateLEDs(void)
{
    static UBYTE currentLED;
    static UBYTE destLED;
    static UBYTE wipeLED = 0;
    static LONG led0Pattern;
    static LONG led1Pattern;
    static LONG flashCountdown;
    UBYTE ledValue;

    /* Display hex values on top LED display */
    for ( ledValue = 0; ledValue <= 0xf; ledValue++ )
    {
        LEDBits( digit_table16_31[ledValue], ledHexTable[ledValue], ON );
    }

    switch( animationMode )
    {
	case 0:
		/* Initialisation state */
		currentLED = 15;
		destLED = 0;
		led0Pattern = 0;
		led1Pattern = 0;
		flashCountdown = 100;
		animationMode = 1;
		break;

	case 1:
		if ( currentLED == destLED ) animationMode = 2;
		else
		{
			/* This animation mode animates each LED from right to left */
			LEDBits( digit_table0_15[currentLED], ledAnim0Table[led0Pattern++], ON );
			if ( led0Pattern == 8 )
			{
				led0Pattern = 0;

				/* Move on to next LED */
				currentLED --;
			}
		}
		break;

	case 2:
		LEDBits( digit_table0_15[destLED], ledAnim1Table[led1Pattern++], ON );
		if ( ledAnim1Table[led1Pattern] == LED_SEG1 )
		{
			destLED ++;
			animationMode = 1;
			currentLED = 15;
		}
		if(destLED == 16)
		{
			animationMode = 3;
		}
		break;

	case 3:
		for ( destLED = 0; destLED < 16; destLED ++ )
		{
			if(flashCountdown % 2)
			{
				LEDBits( digit_table0_15[destLED], ledHexTable[destLED], ON );
			}
			else
			{
				LEDBits( digit_table0_15[destLED], 0, ON );
			}
		}

		flashCountdown --;
		if ( flashCountdown == 0 )
		{
			destLED = 0;
			animationMode = 4;
		}
		break;

	case 4:
		ledValue = ledHexTable[destLED] & ledWipeTable[wipeLED++];
		LEDBits( digit_table0_15[destLED], ledValue, ON );
		if ( wipeLED == 4 )
		{
			wipeLED = 0;
			destLED ++;
			if ( destLED == 16 )
			{
				animationMode = 0;
			}
		}
		break;

	default:
		animationMode = 0;
		break;
    }
}

/************************************************************************/








