/*-------------------------------------------------------------------
Name......... fpga.c
Author....... Marc Adams
Date......... 15/09/03
Description.. Memory mapping and data structures for the Heber
............. reference FPGA
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****	Include Files */

#include	"hardware.h"
#include	"peripherals.h"

#include	"fpga.h"

#include	"fpga.sp"

/****	Local Definitions */
/****	Global Variables */
/****	Local Variables */
/****	Global Constants */

/* Offsets from FPGA_REG_BASE */

/* Audio volume control */
BYTE*const	Xsfxvol=XSFXVOL;

/* FPGA ID */
BYTE*const  FpgaId=FPGA_ID;

/* FPGA Issue */
BYTE*const  FpgaIss=FPGA_ISS;

/* CRC Write */
BYTE*const  CrcCheck=CRCCHECK;
BYTE*const  Xsec=CRCCHECK;


/* Control register */
BYTE*const 	Xcra=XCRA;

/* Multiplexer control register */
BYTE*const 	Xmpx=XMPX;

/* Misc inputs */
BYTE*const 	Xmiscip=XMISC_IP;

/* Audio channels */
BYTE*const 	Sfxr1=SFXR1;
BYTE*const 	Sfxr2=SFXR2;

/* UART routing */
BYTE *const Xduartroute=XDUARTROUTE;

/* UART control register */
BYTE*const	Xuctrl=XUCTRL;

/* Multiplexer base register */
LONG*const	Xmpxbase=MPXBASE;

/* XCRA - control register A */
UBYTE Sfxenable = SFX_ENABLE;
UBYTE Irqenable = IRQ_ENABLE;
UBYTE Mpxenable = MPX_ENABLE;
UBYTE Syncenable = SYNC_ENABLE;

/* XMISC_IP Bits */
UBYTE Metersense = METER_SENSE;
UBYTE Pushbutton = PUSH_BUTTON;

/* FPGA UART base registers */
ULONG FpgaUart0Base = FPGA_UART0_BASE;
ULONG FpgaUart1Base = FPGA_UART1_BASE;

/* Auxillary outputs */
BYTE*const 	Aux0 = AUX0;
BYTE*const 	Aux1 = AUX1;
BYTE*const 	Aux2 = AUX2;
BYTE*const 	Aux3 = AUX3;
BYTE*const 	Aux4 = AUX4;
BYTE*const 	Aux5 = AUX5;
BYTE*const 	Aux6 = AUX6;
BYTE*const 	Aux7 = AUX7;

void*const Outputs=OUTPUTS;
void*const Inputs=INPUTS;


/* Chip Slelect Config */
const CHIPSELECTCFG chipselect_cfg=
{
	CSAR0,		/* Chip Select Address Register 0 */
	CSAMR0,		/* Chip Select Mask Register 0 */
	CSCR0,		/* Chip Select Control Register 0 */
	CSAR1,		/* Chip Select Address Register 1 */
	CSAMR1,		/* Chip Select Mask Register 1 */
	CSCR1,		/* Chip Select Control Register 1 */
	CSAR2,		/* Chip Select Address Register 2 */
	CSAMR2,		/* Chip Select Mask Register 2 */
	CSCR2,		/* Chip Select Control Register 2 */
	CSAR3,		/* Chip Select Address Register 3 */
	CSAMR3,		/* Chip Select Mask Register 3 */
	CSCR3,		/* Chip Select Control Register 3 */
	RAMBAR
};

/* System Integration Module */
const SIMCFG sim_cfg =
{
	0xC0,			/* SIM Config Register */
	0x80,			/* Interrupt Control Register 1 - (auto vector) */
	0x83,			/* Interrupt Control Register 2 */
	0x80,			/* Interrupt Control Register 3 - (auto vector) */
	0x00,			/* Interrupt Control Register 4 */
	0x00,			/* Interrupt Control Register 5 */
	0x00,			/* Interrupt Control Register 6 */
	0x00,			/* Interrupt Control Register 7 */
	0x00,			/* Interrupt Control Register 8 - SWT */
	0x96,			/* ICR9 - Timer 1 (1MS) - (IL5 IP2) */
	0x88,			/* ICR10 - Timer 2 (10MS) - (IL2 IP0) */
	0x00,			/* Interrupt Control Register 11 - MBUS (disabled) */
	0x12,			/* Interrupt Control Register 12 - Uart 1 - (IL4 IP2) */
	0x11,			/* Interrupt Control Register 13 - Uart 2 - (IL2 IP1) */
	0x00,			/* Interrupt Control Register 14 - DMA0 (disabled) */
	0x1b,			/* Interrupt Control Register 15 - DMA1 - (IL6 IP3) */
	0x49f0,			/* Interrupt Mask Register (DMA1,DMA0,Uart2,Uart1,Timer2,Timer1,ICR3,ICR2,ICR1)*/
	0x00,			/* System Protection and Control Register */
	0x00,			/* Softwate Whatch Dog Interruot vector */
	0xFFFC,			/* Pin Assignment Register */
	0x04			/* Bus Arbitration Control */
};

/* DRAM Controller Module */
/* DRAM Controller Module */
const DRAMCFG dram_cfg=
{
    0x0025,         /* DCRR - DRAM CONTROLLER REFRESH REGISTER
                       The DCRR controls the frequency of refresh requests.
                       Refer to Coldfire User's Manual page 11-51 for more
                       details. */
    0x5000,         /* DCTR - DRAM CONTROLLER TIMING REGISTER
                       The DCTR controls the waveform timing for all DRAM
                       transfers.
                       Refer to Coldfire User's Manual page 11-52 for more
                       details. */
    0x6000,         /* DRAM Bank 0 - Base Address */
    0x0FFE0000,     /* DRAM Bank 0 - Address Mask */
    0x93,           /* DRAM Bank 0 - Control Register */
    0x7000,         /* DRAM Bank 1 - Base Address */
    0x0FFE0000,     /* DRAM Bank 1 - Address Mask */
    0x13            /* DRAM Bank 1 - Control Register */
};

/* Timer 1 Module */
const TIMERCFG Timer1Cfg =
{
	0x001b,	/* Timer 1 Mode Register */
	0x9c3f,	/* Timer 1 Reference Register (Set For 1ms Interupt 0xd2ef@54Mhz 0x9C3F@40Mhz*/
	0x0000
};

/* Timer 2 Module */
const TIMERCFG Timer2Cfg =
{
	0xFA1B,	/* Timer 2 Mode Register */
	0x063f,	/* Timer 2 Reference Register (Set For 10ms Interupt 0x086f@54Mhz 0x063f@40Mhz) */
	0x0000
};

/* MBUS Module */
/* NOTE: this is not used - settings are hardcoded in mbusdev.c */
const MBUSCFG mbuscfg =
{
	0x80, /* Address register */
	0x1F, /* Frequency divider -  */
	0x80  /* Control register - Mbus enabled */
};

/****	Local Constants */
/************************************************************************
*****	Functions
*************************************************************************/

/************************************************************************
*****	Options
*************************************************************************/

/************************************************************************
*****	Tasks
*************************************************************************/

/************************************************************************/
